from cone.tile.tests import TileTestCase
from cone.ugm import testing
from cone.ugm.browser.utils import quote_slash
from cone.ugm.browser.utils import unquote_slash


class TestBrowserUtils(TileTestCase):
    layer = testing.ugm_layer

    def test_utils(self):
        quoted = quote_slash('foo/bar')
        self.assertEqual(quoted, 'foo__s_l_a_s_h__bar')
        self.assertEqual(unquote_slash(quoted), 'foo/bar')



from cone.app import get_root
from cone.app.model import BaseNode
from cone.tile import render_tile
from cone.tile.tests import TileTestCase
from cone.ugm import testing
from cone.ugm.model.user import User
from cone.ugm.utils import general_settings


def user_vessel(users):
    vessel = User(BaseNode(), None, None)
    vessel.__parent__ = users
    return vessel


def user_request(layer):
    request = layer.new_request()
    request.params['userform.id'] = ''
    request.params['userform.password'] = '123456'
    request.params['userform.principal_roles'] = []
    request.params['action.userform.save'] = '1'
    return request


class autoincrement_principals(testing.principals):

    def __call__(self, fn):
        w = super(autoincrement_principals, self).__call__(fn)

        def wrapper(inst):
            try:
                w(inst)
            finally:
                settings = general_settings(get_root())
                settings.attrs.user_id_autoincrement = 'False'
                settings.attrs.user_id_autoincrement_prefix = ''
                settings()
        return wrapper


class TestBrowserAutoincrement(TileTestCase):
    layer = testing.ugm_layer

    @autoincrement_principals(
        users={
            'manager': {}
        },
        roles={
            'manager': ['manager']
        }
    )
    def test_autoincrement(self):
        root = get_root()
        users = root['users']

        settings = general_settings(users)
        self.assertEqual(settings.attrs.user_id_autoincrement, 'False')
        self.assertEqual(settings.attrs.user_id_autoincrement_prefix, '')

        vessel = user_vessel(users)
        request = self.layer.new_request()
        with self.layer.authenticated('manager'):
            res = render_tile(vessel, request, 'addform')
        self.checkOutput("""
        ...<input class="form-control required text" id="input-userform-id"
        name="userform.id" required="required" type="text" value="" />...
        """, res)

        settings.attrs.user_id_autoincrement = 'True'
        settings()

        vessel = user_vessel(users)
        with self.layer.authenticated('manager'):
            res = render_tile(vessel, request, 'addform')
        self.checkOutput("""
        ...<input class="form-control text" disabled="disabled"
        id="input-userform-id" name="userform.id" type="text"
        value="auto_incremented" />...
        """, res)

        request = user_request(self.layer)
        vessel = user_vessel(users)
        with self.layer.authenticated('manager'):
            res = render_tile(vessel, request, 'addform')
        self.assertEqual(sorted(users.keys()), ['100', 'manager'])

        request = user_request(self.layer)
        vessel = user_vessel(users)
        with self.layer.authenticated('manager'):
            res = render_tile(vessel, request, 'addform')
        self.assertEqual(sorted(users.keys()), ['100', '101', 'manager'])

        settings.attrs.user_id_autoincrement_prefix = 'uid'
        settings()
        request = user_request(self.layer)
        vessel = user_vessel(users)
        with self.layer.authenticated('manager'):
            res = render_tile(vessel, request, 'addform')
        self.assertEqual(sorted(users.keys()), [
            '100', '101', 'manager', 'uid100'
        ])

        request = user_request(self.layer)
        vessel = user_vessel(users)
        with self.layer.authenticated('manager'):
            res = render_tile(vessel, request, 'addform')
        self.assertEqual(sorted(users.keys()), [
            '100', '101', 'manager', 'uid100', 'uid101'
        ])



from cone.app import get_root
from cone.tile.tests import TileTestCase
from cone.ugm import testing
from pyramid.httpexceptions import HTTPForbidden
from pyramid.view import render_view_to_response
import json


class TestBrowserRemote(TileTestCase):
    layer = testing.ugm_layer

    @testing.principals(
        users={
            'viewer': {},
            'manager': {},
            'user_1': {}
        },
        groups={
            'group_1': {},
            'group_2': {},
        },
        roles={
            'viewer': ['viewer'],
            'manager': ['manager'],
        })
    def test_add_user(self):
        root = get_root()
        users = root['users']
        request = self.layer.new_request(type='json')

        # Need add permission
        with self.layer.authenticated('viewer'):
            self.expectError(
                HTTPForbidden,
                render_view_to_response,
                users,
                request,
                name='remote_add_user'
            )

        # No id given
        request.params['id'] = ''
        with self.layer.authenticated('manager'):
            res = render_view_to_response(users, request, name='remote_add_user')
        self.assertEqual(json.loads(res.text), {
            'message': 'No user ID given.',
            'success': False
        })

        # Existent id given
        request.params['id'] = 'user_1'
        with self.layer.authenticated('manager'):
            res = render_view_to_response(users, request, name='remote_add_user')
        self.assertEqual(json.loads(res.text), {
            'message': 'User with given ID already exists.',
            'success': False
        })

        # Add minimal valid user
        request.params['id'] = 'user_2'
        request.params['attr.fullname'] = 'Sepp Unterwurzacher'
        request.params['attr.email'] = 'sepp.unterwurzacher@example.com'
        with self.layer.authenticated('manager'):
            res = render_view_to_response(users, request, name='remote_add_user')
        self.assertEqual(json.loads(res.text), {
            'message': "Created user with ID 'user_2'.",
            'success': True
        })

        # Check new user
        user = users['user_2']

        self.assertEqual(user.attrs['fullname'], 'Sepp Unterwurzacher')
        self.assertEqual(user.attrs['email'], 'sepp.unterwurzacher@example.com')

        # This user has nor roles and is not member of a group
        self.assertEqual(user.model.roles, [])
        self.assertEqual(user.model.groups, [])

        # There was no password given, thus we cannot authenticate with this
        # user yet
        self.assertFalse(user.model.authenticate('secret'))

        user.model.passwd(None, 'secret')
        self.assertTrue(user.model.authenticate('secret'))

        # Create another user with initial password
        request.params['id'] = 'user_3'
        request.params['password'] = 'secret'
        with self.layer.authenticated('manager'):
            res = render_view_to_response(users, request, name='remote_add_user')
        self.assertEqual(json.loads(res.text), {
            'message': "Created user with ID 'user_3'.",
            'success': True
        })

        user = users['user_3']
        self.assertTrue(user.model.authenticate('secret'))

        # Create user with initial roles. Message tells us if some of this
        # roles are not available
        request.params['id'] = 'user_4'
        request.params['password'] = 'secret'
        request.params['roles'] = 'editor,viewer,inexistent'
        with self.layer.authenticated('manager'):
            res = render_view_to_response(users, request, name='remote_add_user')
        self.assertEqual(json.loads(res.text), {
            'message': "Role 'inexistent' given but inexistent. Created user with ID 'user_4'.",
            'success': True
        })

        # Create user with intial group membership. Message tells us if some of
        # this groups are not available
        request.params['id'] = 'user_5'
        request.params['password'] = 'secret'
        request.params['roles'] = 'editor,viewer,inexistent'
        request.params['groups'] = 'group_1,group_2,group_inexistent'
        with self.layer.authenticated('manager'):
            res = render_view_to_response(users, request, name='remote_add_user')
        self.assertEqual(json.loads(res.text), {
            'message': (
                "Role 'inexistent' given but inexistent. Group 'group_inexistent' "
                "given but inexistent. Created user with ID 'user_5'."
            ),
            'success': True
        })

        # Check created user
        user = users['user_5']
        self.assertEqual(sorted(user.model.group_ids), ['group_1', 'group_2'])
        self.assertEqual(sorted(user.model.roles), ['editor', 'viewer'])
        self.assertTrue(user.model.authenticate('secret'))

    @testing.principals(
        users={
            'viewer': {},
            'manager': {},
            'user_1': {}
        },
        roles={
            'viewer': ['viewer'],
            'manager': ['manager'],
        })
    def test_delete_user(self):
        root = get_root()
        users = root['users']

        request = self.layer.new_request(type='json')

        with self.layer.authenticated('viewer'):
            self.expectError(
                HTTPForbidden,
                render_view_to_response,
                users,
                request,
                name='remote_delete_user'
            )

        # No id given
        request.params['id'] = ''
        with self.layer.authenticated('manager'):
            res = render_view_to_response(users, request, name='remote_delete_user')
        self.assertEqual(json.loads(res.text), {
            'message': 'No user ID given.',
            'success': False
        })

        # Inexistent id given
        request.params['id'] = 'user_2'
        with self.layer.authenticated('manager'):
            res = render_view_to_response(users, request, name='remote_delete_user')
        self.assertEqual(json.loads(res.text), {
            'message': 'User with given ID not exists.',
            'success': False
        })

        # Valid deletions
        request.params['id'] = 'user_1'
        with self.layer.authenticated('manager'):
            res = render_view_to_response(users, request, name='remote_delete_user')
        self.assertEqual(json.loads(res.text), {
            'message': "Deleted user with ID 'user_1\'.",
            'success': True
        })



from cone.app import get_root
from cone.tile import render_tile
from cone.tile.tests import TileTestCase
from cone.ugm import testing
from cone.ugm.utils import general_settings
from io import BytesIO
import pkg_resources


def user_portrait_request(layer, model, portrait):
    request = layer.new_request()
    request.params['ajax'] = '1'
    request.params['userform.password'] = '_NOCHANGE_'
    request.params['userform.portrait'] = portrait
    request.params['userform.principal_roles'] = []
    request.params['action.userform.save'] = '1'
    return request


def dummy_file_data(filename):
    path = pkg_resources.resource_filename(
        'yafowil.widget.image',
        'testing/{}'.format(filename)
    )
    with open(path, 'rb') as file:
        data = file.read()
    return data


class portrait_principals(testing.principals):

    def __call__(self, fn):
        w = super(portrait_principals, self).__call__(fn)

        def wrapper(inst):
            try:
                w(inst)
            finally:
                settings = general_settings(get_root())
                settings.attrs.users_portrait = u'True'
                settings()
        return wrapper


class TestBrowserPortrait(TileTestCase):
    layer = testing.ugm_layer

    @portrait_principals(
        users={
            'manager': {},
            'user_1': {}
        },
        roles={
            'manager': ['manager']
        })
    def test_portrait(self):
        root = get_root()
        users = root['users']
        user = users['user_1']

        # Portrait related config properties
        settings = general_settings(users)
        self.assertEqual(settings.attrs.users_portrait, 'True')
        self.assertEqual(settings.attrs.users_portrait_attr, 'portrait')
        self.assertEqual(settings.attrs.users_portrait_accept, 'image/jpeg')
        self.assertEqual(settings.attrs.users_portrait_width, '50')
        self.assertEqual(settings.attrs.users_portrait_height, '50')

        # Portrait enabled, widget is rendered
        request = self.layer.new_request()
        with self.layer.authenticated('manager'):
            res = render_tile(user, request, 'editform')
        self.assertTrue(res.find('id="input-userform-portrait"') > -1)

        # No portrait, default portrait is shown
        expected = (
            'src="http://example.com/cone.ugm.static/images/'
            'default_portrait.jpg?nocache='
        )
        self.assertTrue(res.find(expected) > -1)

        # Submit portrait
        dummy_jpg = dummy_file_data('dummy.jpg')
        portrait = {
            'file': BytesIO(dummy_jpg),
            'mimetype': 'image/jpeg',
        }

        request = user_portrait_request(self.layer, user, portrait)
        with self.layer.authenticated('manager'):
            res = render_tile(user, request, 'editform')

        # New portrait set on user
        expected = b'\xff\xd8\xff\xe0\x00\x10JFIF'
        self.assertTrue(user.attrs['portrait'].startswith(expected))

        # Portrait present, link to user portrait is shown
        request = self.layer.new_request()
        with self.layer.authenticated('manager'):
            res = render_tile(user, request, 'editform')
        expected = 'src="http://example.com/users/user_1/portrait_image?nocache='
        self.assertTrue(res.find(expected) > -1)

        # Portrait disabled, widget is skipped
        settings.attrs.users_portrait = u'False'
        settings()

        request = self.layer.new_request()
        with self.layer.authenticated('manager'):
            res = render_tile(user, request, 'editform')
        self.assertFalse(res.find('id="input-userform-portrait"') > -1)



from cone.app.model import AppNode
from cone.app.model import Metadata
from cone.app.model import node_info
from cone.app.model import Properties
from cone.app.ugm import ugm_backend
from cone.ugm.browser.utils import unquote_slash
from cone.ugm.localmanager import LocalManagerGroupsACL
from cone.ugm.model.group import Group
from node.behaviors import Nodify
from node.locking import locktree
from node.utils import instance_property
from plumber import plumbing
from pyramid.i18n import TranslationStringFactory
import logging


logger = logging.getLogger('cone.ugm')
_ = TranslationStringFactory('cone.ugm')


@node_info(
    'groups',
    title=_('groups_node', default='Groups'),
    description=_(
        'groups_node_description',
        default='Container for Groups'),
    icon='ion-person-stalker',
    addables=['group'])
@plumbing(
    LocalManagerGroupsACL,
    AppNode,
    Nodify)
class Groups(object):

    @instance_property
    def properties(self):
        props = Properties()
        props.in_navtree = True
        return props

    @instance_property
    def metadata(self):
        metadata = Metadata()
        metadata.title = _('groups_node', default='Groups')
        metadata.description = _(
            'groups_node_description',
            default='Container for Groups'
        )
        return metadata

    @property
    def backend(self):
        return ugm_backend.ugm.groups

    @locktree
    def invalidate(self, key=None):
        if key is None:
            self.backend.parent.invalidate('groups')
            return
        self.backend.invalidate(key)

    @locktree
    def __call__(self):
        self.backend()

    @locktree
    def __iter__(self):
        try:
            return self.backend.__iter__()
        except Exception as e:
            logger.error(str(e))
            return iter(list())

    iterkeys = __iter__

    @locktree
    def __getitem__(self, name):
        # XXX: temporary hack until paster/webob/pyramid handle urllib
        # quoted slashes in path components
        name = unquote_slash(name)
        try:
            model = self.backend[name]
        except AttributeError:
            raise KeyError(name)
        return Group(model, name, self)



from cone.app import get_root
from cone.tile import render_tile
from cone.tile.tests import TileTestCase
from cone.ugm import testing
from cone.ugm.model.user import User
from pyramid.httpexceptions import HTTPForbidden
from webob.exc import HTTPFound


class TestBrowserUser(TileTestCase):
    layer = testing.ugm_layer

    @testing.principals(
        users={
            'user_1': {}
        })
    def test_content_tile(self):
        root = get_root()
        users = root['users']
        user = users['user_1']
        request = self.layer.new_request()

        # Unauthenticated content tile renders login form
        expected = '<form action="http://example.com/users/user_1/login"'
        res = render_tile(user, request, 'content')
        self.assertTrue(res.find(expected) > -1)

    @testing.principals(
        users={
            'manager': {},
            'user_1': {}
        },
        roles={
            'manager': ['manager']
        })
    def test_leftcolumn_tile(self):
        root = get_root()
        users = root['users']
        user = users['user_1']
        request = self.layer.new_request()

        self.expectError(
            HTTPForbidden,
            render_tile,
            user,
            request,
            'leftcolumn'
        )

        with self.layer.authenticated('manager'):
            res = render_tile(user, request, 'leftcolumn')
        expected = '<div class="column left_column col-md-6">'
        self.assertTrue(res.find(expected) > -1)

    @testing.principals(
        users={
            'manager': {},
            'user_1': {}
        },
        roles={
            'manager': ['manager']
        })
    def test_rightcolumn_tile(self):
        root = get_root()
        users = root['users']
        user = users['user_1']
        request = self.layer.new_request()

        self.expectError(
            HTTPForbidden,
            render_tile,
            user,
            request,
            'rightcolumn'
        )

        with self.layer.authenticated('manager'):
            res = render_tile(user, request, 'rightcolumn')
        expected = '<div class="column right_column col-md-6">'
        self.assertTrue(res.find(expected) > -1)

    @testing.principals(
        users={
            'manager': {},
            'user_1': {}
        },
        groups={
            'group_1': {}
        },
        membership={
            'group_1': ['user_1']
        },
        roles={
            'manager': ['manager']
        })
    def test_columnlisting_tile(self):
        root = get_root()
        users = root['users']
        user = users['user_1']
        request = self.layer.new_request()

        self.expectError(
            HTTPForbidden,
            render_tile,
            user,
            request,
            'columnlisting'
        )

        with self.layer.authenticated('manager'):
            res = render_tile(user, request, 'columnlisting')
        expected = (
            '<li class="list-group-item "\n              '
            'ajax:target="http://example.com/groups/group_1">'
        )
        self.assertTrue(res.find(expected) > -1)

    @testing.principals(
        users={
            'manager': {},
            'user_1': {}
        },
        groups={
            'group_1': {},
            'group_2': {}

        },
        membership={
            'group_1': ['user_1']
        },
        roles={
            'manager': ['manager']
        })
    def test_allcolumnlisting_tile(self):
        root = get_root()
        users = root['users']
        user = users['user_1']
        request = self.layer.new_request()

        self.expectError(
            HTTPForbidden,
            render_tile,
            user,
            request,
            'allcolumnlisting'
        )

        with self.layer.authenticated('manager'):
            res = render_tile(user, request, 'allcolumnlisting')
        expected = (
            '<li class="list-group-item "\n              '
            'ajax:target="http://example.com/groups/group_1">'
        )
        self.assertTrue(res.find(expected) > -1)

        expected = (
            '<li class="list-group-item "\n              '
            'ajax:target="http://example.com/groups/group_2">'
        )
        self.assertTrue(res.find(expected) > -1)

    @testing.principals(
        users={
            'manager': {}
        },
        roles={
            'manager': ['manager']
        })
    def test_add_user(self):
        root = get_root()
        users = root['users']
        request = self.layer.new_request()
        request.params['factory'] = 'user'

        with self.layer.authenticated('viewer'):
            self.expectError(
                HTTPForbidden,
                render_tile,
                users,
                request,
                'add'
            )

        with self.layer.authenticated('manager'):
            res = render_tile(users, request, 'add')
        expected = '<form action="http://example.com/users/add"'
        self.assertTrue(res.find(expected) > -1)

        request.params['userform.id'] = ''
        request.params['userform.password'] = 'secret_1'
        request.params['userform.fullname'] = 'Max Mustermann'
        request.params['userform.email'] = 'max.mustermann@example.com'
        request.params['userform.principal_roles'] = []
        request.params['action.userform.save'] = '1'

        with self.layer.authenticated('manager'):
            res = render_tile(users, request, 'add')
        expected = '<div class="text-danger">No User ID defined</div>'
        self.assertTrue(res.find(expected) > -1)

        request.params['userform.id'] = 'user_1'

        with self.layer.authenticated('manager'):
            res = render_tile(users, request, 'add')
        self.assertEqual(res, '')
        self.assertTrue(isinstance(request.environ['redirect'], HTTPFound))
        self.assertEqual(sorted(users.keys()), ['manager', 'user_1'])

        user = users['user_1']
        self.assertTrue(isinstance(user, User))
        self.assertTrue(bool(user.attrs['password']))
        self.assertEqual(user.attrs['fullname'], 'Max Mustermann')
        self.assertEqual(user.attrs['email'], 'max.mustermann@example.com')

    @testing.principals(
        users={
            'manager': {},
            'user_1': {
                'fullname': 'Max Mustermann',
                'email': 'max.mustermann@example.com'
            }
        },
        roles={
            'manager': ['manager']
        })
    def test_edit_user(self):
        root = get_root()
        users = root['users']
        user = users['user_1']
        self.assertEqual(user.attrs['fullname'], 'Max Mustermann')
        self.assertEqual(user.attrs['email'], 'max.mustermann@example.com')

        request = self.layer.new_request()

        with self.layer.authenticated('manager'):
            res = render_tile(user, request, 'edit')
        expected = '<form action="http://example.com/users/user_1/edit"'
        self.assertTrue(res.find(expected) > -1)

        request.params['userform.password'] = '_NOCHANGE_'
        request.params['userform.fullname'] = 'Susi Musterfrau'
        request.params['userform.email'] = 'susi.musterfrau@example.com'
        request.params['userform.principal_roles'] = []
        request.params['action.userform.save'] = '1'
        with self.layer.authenticated('manager'):
            res = render_tile(user, request, 'edit')
        self.assertEqual(res, '')

        self.assertEqual(user.attrs['fullname'], 'Susi Musterfrau')
        self.assertEqual(user.attrs['email'], 'susi.musterfrau@example.com')



from cone.app import root
from cone.app.model import Metadata
from cone.app.model import Properties
from cone.ugm import testing
from cone.ugm import ugm_default_acl
from cone.ugm.model.groups import Group
from cone.ugm.model.groups import Groups
from node.ext.ugm.interfaces import IGroups
from node.tests import NodeTestCase


class TestModelGroups(NodeTestCase):
    layer = testing.ugm_layer

    @testing.principals(
        groups={
            'group_1': {},
            'group_2': {},
        })
    def test_groups(self):
        # Groups container
        groups = root['groups']
        self.assertTrue(isinstance(groups, Groups))
        self.assertEqual(groups.name, 'groups')

        # Properties
        self.assertTrue(isinstance(groups.properties, Properties))

        # Metadata
        md = groups.metadata
        self.assertTrue(isinstance(md, Metadata))
        self.assertEqual(md.title, 'groups_node')
        self.assertEqual(md.description, 'groups_node_description')

        # Iter groups
        self.assertEqual(len([x for x in groups]), 2)

        # Inexistent child
        self.expect_error(KeyError, groups.__getitem__, 'inexistent')

        # Children are group application nodes
        group = groups['group_1']
        self.assertTrue(isinstance(group, Group))

        # Check UGM backend
        backend = groups.backend
        self.assertTrue(IGroups.providedBy(backend))

        # Check invalidate
        self.assertTrue(backend is groups.backend)
        groups.invalidate()
        self.assertFalse(backend is groups.backend)

        # ACL
        self.assertEqual(groups.__acl__, ugm_default_acl)



from cone.tile import tile, Tile
from cone.app.model import AppRoot
from cone.ugm.browser.columns import Column


@tile(name='site', interface=AppRoot, permission='view')
class SiteName(Tile):

    def render(self):
        # XXX: remove
        return 'SITENAME'


@tile(name='leftcolumn', interface=AppRoot, permission='view')
class RootLeftColumn(Column):

    def render(self):
        return self._render(self.model['users'], 'leftcolumn')


@tile(name='rightcolumn', interface=AppRoot, permission='view')
class RootRightColumn(Column):

    def render(self):
        return self._render(self.model['users'], 'rightcolumn')



from cone.app.browser.layout import ProtectedContentTile
from cone.tile import render_tile
from cone.tile import tile
from cone.tile import Tile
from cone.ugm.model.group import Group
from cone.ugm.model.groups import Groups
from cone.ugm.model.user import User
from cone.ugm.model.users import Users


@tile(
    name='content',
    path='cone.ugm:browser/templates/columns.pt',
    interface=Group,
    permission='login',
    strict=False)
@tile(
    name='content',
    path='cone.ugm:browser/templates/columns.pt',
    interface=Groups,
    permission='login',
    strict=False)
@tile(
    name='content',
    path='cone.ugm:browser/templates/columns.pt',
    interface=User,
    permission='login',
    strict=False)
@tile(
    name='content',
    path='cone.ugm:browser/templates/columns.pt',
    interface=Users,
    permission='login',
    strict=False)
class Columns(ProtectedContentTile):
    pass


class Column(Tile):
    """Abstract column.
    """

    def _render(self, model, name):
        return render_tile(model, self.request, name)



from cone.app import compat
from cone.app.browser.ajax import AjaxAction
from cone.app.browser.authoring import ContentAddForm
from cone.app.browser.authoring import ContentEditForm
from cone.app.browser.form import Form
from cone.app.browser.utils import make_query
from cone.app.browser.utils import make_url
from cone.app.ugm import ugm_backend
from cone.tile import Tile
from cone.tile import tile
from cone.ugm.browser.authoring import AddFormFiddle
from cone.ugm.browser.authoring import EditFormFiddle
from cone.ugm.browser.listing import ColumnListing
from cone.ugm.browser.principal import PrincipalForm
from cone.ugm.browser.principal import group_field
from cone.ugm.browser.roles import PrincipalRolesForm
from cone.ugm.model.group import Group
from cone.ugm.utils import general_settings
from odict import odict
from plumber import plumbing
from pyramid.i18n import TranslationStringFactory
from webob.exc import HTTPFound
import fnmatch
import itertools


_ = TranslationStringFactory('cone.ugm')


@tile(
    name='leftcolumn',
    path='templates/principal_left_column.pt',
    interface=Group,
    permission='view')
class GroupLeftColumn(Tile):

    @property
    def principals_target(self):
        query = make_query(pid=self.model.name)
        return make_url(self.request, node=self.model.parent, query=query)


@tile(
    name='rightcolumn',
    path='templates/principal_right_column.pt',
    interface=Group,
    permission='view')
class GroupRightColumn(Tile):
    pass


class UsersListing(ColumnListing):
    members_only = False

    @property
    def listing_items(self):
        appgroup = self.model
        group = appgroup.model
        member_ids = group.keys()
        # Always True if we list members only, otherwise will be set
        # in the loop below
        related = self.members_only
        # XXX: so far only users as members of groups, for
        # group-in-group we need to prefix groups
        if related:
            users = group.users
        else:
            # XXX: speedup
            users = group.root.users.values()
        # reduce for local manager
        if self.model.local_manager_consider_for_user:
            local_uids = self.model.local_manager_target_uids
            users = [u for u in users if u.name in local_uids]
        attrlist = self.user_attrs
        sort_attr = self.user_default_sort_column
        filter_term = self.unquoted_param_value('filter')
        can_change = self.request.has_permission(
            'manage_membership',
            self.model.parent
        )
        ret = list()
        for user in users:
            attrs = user.attrs
            # reduce by search term
            if filter_term:
                s_attrs = [attrs[attr] for attr in attrlist]
                if not fnmatch.filter(s_attrs, filter_term):
                    continue
            uid = user.name
            item_target = make_url(self.request, path=user.path[1:])
            action_query = make_query(id=uid)
            action_target = make_url(
                self.request,
                node=appgroup,
                query=action_query
            )
            if not self.members_only:
                related = uid in member_ids
            actions = list()
            if can_change:
                action_id = 'add_item'
                action_enabled = not bool(related)
                action_title = _(
                    'add_user_to_selected_group',
                    default='Add user to selected group'
                )
                add_item_action = self.create_action(
                    action_id,
                    action_enabled,
                    action_title,
                    action_target
                )
                actions.append(add_item_action)
                action_id = 'remove_item'
                action_enabled = bool(related)
                action_title = _(
                    'remove_user_from_selected_group',
                    default='Remove user from selected group'
                )
                remove_item_action = self.create_action(
                    action_id,
                    action_enabled,
                    action_title,
                    action_target
                )
                actions.append(remove_item_action)
            vals = [self.extract_raw(attrs, attr) for attr in attrlist]
            sort = self.extract_raw(attrs, sort_attr)
            content = self.item_content(*vals)
            current = False
            item = self.create_item(
                sort,
                item_target,
                content,
                current,
                actions
            )
            ret.append(item)
        return ret


@tile(
    name='columnlisting',
    path='templates/column_listing.pt',
    interface=Group,
    permission='view')
class UsersOfGroupColumnListing(UsersListing):
    css = 'users'
    slot = 'rightlisting'
    list_columns = ColumnListing.user_list_columns
    members_only = True
    batchname = 'rightbatch'
    display_limit = True
    display_limit_checked = False


@tile(
    name='allcolumnlisting',
    path='templates/column_listing.pt',
    interface=Group,
    permission='view')
class AllUsersColumnListing(UsersListing):
    css = 'users'
    slot = 'rightlisting'
    list_columns = ColumnListing.user_list_columns
    batchname = 'rightbatch'
    display_limit = True
    display_limit_checked = True

    @property
    def ajax_action(self):
        return 'allcolumnlisting'


class GroupForm(PrincipalForm):
    form_name = 'groupform'
    field_factory_registry = group_field

    @property
    def reserved_attrs(self):
        return odict([
            ('id', _('group_id', default='Group ID'))
        ])

    @property
    def form_attrmap(self):
        return general_settings(self.model).attrs.groups_form_attrmap


@tile(name='addform', interface=Group, permission="add_group")
@plumbing(ContentAddForm, PrincipalRolesForm, AddFormFiddle)
class GroupAddForm(GroupForm, Form):
    show_heading = False
    show_contextmenu = False

    def save(self, widget, data):
        extracted = dict()
        for attr_name in itertools.chain(self.reserved_attrs, self.form_attrmap):
            value = data[attr_name].extracted
            if not value:
                continue
            extracted[attr_name] = value
        groups = ugm_backend.ugm.groups
        group_id = extracted.pop('id')
        groups.create(group_id, **extracted)
        groups()
        self.request.environ['next_resource'] = group_id
        self.model.parent.invalidate()

    def next(self, request):
        next_resource = self.request.environ.get('next_resource')
        if next_resource:
            query = make_query(pid=next_resource)
            url = make_url(request.request, node=self.model.parent, query=query)
        else:
            url = make_url(request.request, node=self.model.parent)
        if self.ajax_request:
            return [
                AjaxAction(url, 'leftcolumn', 'replace', '.left_column'),
                AjaxAction(url, 'rightcolumn', 'replace', '.right_column'),
            ]
        return HTTPFound(location=url)


@tile(name='editform', interface=Group, permission="edit_group", strict=False)
@plumbing(ContentEditForm, PrincipalRolesForm, EditFormFiddle)
class GroupEditForm(GroupForm, Form):
    show_heading = False
    show_contextmenu = False

    def save(self, widget, data):
        attrs = self.model.attrs
        for attr_name in self.form_attrmap:
            if attr_name in ['id']:
                continue
            attrs[attr_name] = data[attr_name].extracted
        self.model()

    def next(self, request):
        came_from = request.get('came_from')
        if came_from:
            came_from = compat.unquote(came_from)
            url = '{}?pid={}'.format(came_from, self.model.name)
        else:
            url = make_url(request.request, node=self.model)
        if self.ajax_request:
            return [
                AjaxAction(url, 'leftcolumn', 'replace', '.left_column'),
                AjaxAction(url, 'rightcolumn', 'replace', '.right_column'),
            ]
        return HTTPFound(location=url)



from cone.ugm.utils import general_settings
from plumber import Behavior
from plumber import default
from plumber import plumb
from pyramid.i18n import TranslationStringFactory
from yafowil.base import factory


_ = TranslationStringFactory('cone.ugm')


class PrincipalRolesForm(Behavior):

    @default
    @property
    def roles_vocab(self):
        from cone.app.security import DEFAULT_ROLES
        return DEFAULT_ROLES

    @default
    @property
    def roles_enabled(self):
        settings = general_settings(self.model).attrs
        return settings.roles_principal_roles_enabled == 'True'

    @plumb
    def prepare(_next, self):
        """Hook after prepare and set 'principal_roles' as selection to
        ``self.form``.
        """
        _next(self)
        if not self.roles_enabled:
            return
        if not self.request.has_permission('manage', self.model.parent):
            # XXX: yafowil selection display renderer
            return
        value = []
        if self.action_resource == 'edit':
            value = self.model.model.roles
        roles_widget = factory(
            'field:label:select',
            name='principal_roles',
            value=value,
            props={
                'label': _('roles', default='Roles'),
                'multivalued': True,
                'vocabulary': self.roles_vocab,
                'format': 'single',
                'listing_tag': 'ul',
                'listing_label_position': 'after',
            })
        save_widget = self.form['save']
        self.form.insertbefore(roles_widget, save_widget)

    @plumb
    def save(_next, self, widget, data):
        _next(self, widget, data)
        if not self.roles_enabled:
            return
        if not self.request.has_permission('manage', self.model.parent):
            return
        form_name = self.form_name
        existing_roles = list()
        if self.action_resource == 'edit':
            principal = self.model.model
            existing_roles = principal.roles
        else:
            uid = data.fetch('{}.id'.format(form_name)).extracted
            principal = self.model.parent[uid].model
        new_roles = data.fetch('{}.principal_roles'.format(form_name)).extracted
        removed_roles = list()
        for role in existing_roles:
            if role not in new_roles:
                principal.remove_role(role)
                removed_roles.append(role)
        for role in removed_roles:
            existing_roles.remove(role)
        for role in new_roles:
            if role not in existing_roles:
                principal.add_role(role)
        principal.parent.parent()



from cone.app import security
from cone.ugm.utils import general_settings
from cone.ugm.utils import localmanager_settings
from lxml import etree
from node.behaviors import DictStorage
from node.behaviors import Nodify
from plumber import Behavior
from plumber import default
from plumber import finalize
from plumber import plumb
from plumber import plumbing
from pyramid.security import Allow
from pyramid.threadlocal import get_current_request
import os


class LocalManagerConfig(DictStorage):
    """Local Management configuration storage.
    """

    @finalize
    def load(self):
        path = self.file_path
        if not path or not os.path.exists(path):
            return
        with open(path, 'r') as handle:
            tree = etree.parse(handle)
        root = tree.getroot()
        for rule in root.getchildren():
            new_rule = self.storage[rule.tag] = dict()
            for prop in rule.getchildren():
                for tag_name in ['target', 'default']:
                    if prop.tag == tag_name:
                        new_rule[tag_name] = list()
                        for group in prop.getchildren():
                            new_rule[tag_name].append(group.text)

    @finalize
    def __call__(self):
        root = etree.Element('localmanager')
        for gid, rule in self.storage.items():
            group = etree.SubElement(root, gid)
            for tag_name in ['target', 'default']:
                elem = etree.SubElement(group, tag_name)
                for gid in rule[tag_name]:
                    item = etree.SubElement(elem, 'item')
                    item.text = gid
        with open(self.file_path, 'wb') as handle:
            handle.write(etree.tostring(root, pretty_print=True))


@plumbing(Nodify, LocalManagerConfig)
class LocalManagerConfigAttributes(object):

    def __init__(self, path):
        self.file_path = path
        self.load()


class LocalManager(Behavior):
    """Behavior providing local manager information for authenticated user.
    """

    @finalize
    @property
    def local_management_enabled(self):
        """Flag whether local management is enabled.
        """
        settings = general_settings(self.root)
        return settings.attrs.users_local_management_enabled == 'True'

    @finalize
    @property
    def local_manager_consider_for_user(self):
        """Flag whether local manager ACL should be considered for current
        authenticated user.
        """
        if not self.local_management_enabled:
            return False
        request = get_current_request()
        if request.authenticated_userid == security.ADMIN_USER:
            return False
        roles = security.authenticated_user(request).roles
        if 'admin' in roles or 'manager' in roles:
            return False
        return True

    @finalize
    @property
    def local_manager_gid(self):
        """Group id of local manager group of current authenticated member.

        Currently a user can be assigned only to one local manager group. If
        more than one local manager group is configured, an error is raised.
        """
        settings = localmanager_settings(self.root)
        user = security.authenticated_user(get_current_request())
        if not user:
            return None
        gids = user.group_ids
        adm_gids = list()
        for gid in gids:
            rule = settings.attrs.get(gid)
            if rule:
                adm_gids.append(gid)
        if len(adm_gids) == 0:
            return None
        if len(adm_gids) > 1:
            msg = (
                u"Authenticated member defined in local manager "
                u"groups %s but only one management group allowed for "
                u"each user. Please contact System Administrator in "
                u"order to fix this problem."
            )
            exc = msg % ', '.join(["'%s'" % gid for gid in sorted(adm_gids)])
            raise Exception(exc)
        return adm_gids[0]

    @finalize
    @property
    def local_manager_rule(self):
        """Return rule for local manager.
        """
        adm_gid = self.local_manager_gid
        if not adm_gid:
            return None
        settings = localmanager_settings(self.root)
        return settings.attrs[adm_gid]

    @finalize
    @property
    def local_manager_default_gids(self):
        """Return default group id's for local manager.
        """
        rule = self.local_manager_rule
        if not rule:
            return list()
        return rule['default']

    @finalize
    @property
    def local_manager_target_gids(self):
        """Target group id's for local manager.
        """
        rule = self.local_manager_rule
        if not rule:
            return list()
        return rule['target']

    @finalize
    @property
    def local_manager_target_uids(self):
        """Target uid's for local manager.
        """
        groups = self.root['groups'].backend
        managed_uids = set()
        for gid in self.local_manager_target_gids:
            group = groups.get(gid)
            if group:
                managed_uids.update(group.member_ids)
        return list(managed_uids)

    @finalize
    def local_manager_is_default(self, adm_gid, gid):
        """Check whether gid is default group for local manager group.
        """
        settings = localmanager_settings(self.root)
        rule = settings.attrs[adm_gid]
        if gid not in rule['target']:
            raise Exception(u"group '%s' not managed by '%s'" % (gid, adm_gid))
        return gid in rule['default']


class LocalManagerACL(LocalManager):
    """Behavior providing ACL's by local manager configuration.
    """

    @default
    @property
    def local_manager_acl(self):
        raise NotImplementedError(u"Abstract ``LocalManagerACL`` does not "
                                  u"implement ``local_manager_acl``")

    @plumb
    @property
    def __acl__(_next, self):
        acl = _next(self)
        if not self.local_manager_consider_for_user:
            return acl
        return self.local_manager_acl + acl


class LocalManagerUsersACL(LocalManagerACL):

    @finalize
    @property
    def local_manager_acl(self):
        if not self.local_manager_target_gids:
            return []
        permissions = [
            'view', 'add', 'add_user', 'edit', 'edit_user',
            'manage_expiration', 'manage_membership'
        ]
        return [(
            Allow,
            get_current_request().authenticated_userid,
            permissions
        )]


class LocalManagerUserACL(LocalManagerACL):

    @finalize
    @property
    def local_manager_acl(self):
        # if self.name is None, User object was created by add model factory
        if self.name is not None:
            if self.name not in self.local_manager_target_uids:
                return []
        permissions = [
            'view', 'add', 'add_user', 'edit', 'edit_user',
            'manage_expiration', 'manage_membership'
        ]
        return [(
            Allow,
            get_current_request().authenticated_userid,
            permissions
        )]


class LocalManagerGroupsACL(LocalManagerACL):

    @finalize
    @property
    def local_manager_acl(self):
        if not self.local_manager_target_gids:
            return []
        permissions = ['view', 'manage_membership']
        return [(
            Allow,
            get_current_request().authenticated_userid,
            permissions
        )]


class LocalManagerGroupACL(LocalManagerACL):

    @finalize
    @property
    def local_manager_acl(self):
        if self.name not in self.local_manager_target_gids:
            return []
        permissions = ['view', 'manage_membership']
        return [(
            Allow,
            get_current_request().authenticated_userid,
            permissions
        )]



from cone.tile.tests import TileTestCase
from cone.ugm import testing
from node.utils import UNSET
from yafowil.base import factory


class TestBrowserExpires(TileTestCase):
    layer = testing.ugm_layer

    def test_expiration_widget(self):
        # Edit renderer. Active with no expiration date by default
        widget = factory(
            'expiration',
            name='active',
            props={
                'datepicker': True,
                'locale': 'de',
            })
        self.checkOutput("""
        <div class="expiration-widget"><input checked="checked"
        id="checkbox-active" name="active.active" type="checkbox"
        value="1" /><label>until</label><input class="datepicker expiration
        form-control" id="input-active" name="active" size="10" type="text"
        value="" /></div>
        """, widget())

        # Active with no expiration date
        request = dict()
        request['active.active'] = '1'
        request['active'] = ''
        data = widget.extract(request)
        self.assertEqual(data.extracted, UNSET)

        self.checkOutput("""
        <div class="expiration-widget"><input checked="checked"
        id="checkbox-active" name="active.active" type="checkbox"
        value="1" /><label>until</label><input class="datepicker expiration
        form-control" id="input-active" name="active" size="10" type="text"
        value="" /></div>
        """, widget(request=request))

        # Inactive
        request = dict()
        request['active.active'] = '0'
        request['active'] = ''
        data = widget.extract(request)
        self.assertEqual(data.extracted, 0)

        # Active with expiration date
        request = dict()
        request['active.active'] = '1'
        request['active'] = '23.12.2012'
        data = widget.extract(request)
        self.assertEqual(data.extracted, 1356217200.0)

        # Edit renderer with preset value
        widget = factory(
            'expiration',
            name='active',
            value=0,
            props={
                'datepicker': True,
                'locale': 'de',
            })
        self.checkOutput("""
        <div class="expiration-widget"><input id="checkbox-active"
        name="active.active" type="checkbox" value="1" /><label>until</label><input
        class="datepicker expiration form-control" id="input-active" name="active"
        size="10" type="text" value="" /></div>
        """, widget())

        widget = factory(
            'expiration',
            name='active',
            value=1356217200.0,
            props={
                'datepicker': True,
                'locale': 'de',
            })
        self.checkOutput("""
        <div class="expiration-widget"><input checked="checked"
        id="checkbox-active" name="active.active" type="checkbox"
        value="1" /><label>until</label><input class="datepicker expiration
        form-control" id="input-active" name="active" size="10" type="text"
        value="23.12.2012" /></div>
        """, widget())

        # Display renderer
        widget = factory(
            'expiration',
            name='active',
            props={
                'datepicker': True,
                'locale': 'de',
            },
            mode='display')
        self.checkOutput("""
        <div class="expiration-widget"><input checked="checked"
        disabled="disabled" id="checkbox-active"
        type="checkbox" /></div>
        """, widget())

        widget = factory(
            'expiration',
            name='active',
            value=0,
            props={
                'datepicker': True,
                'locale': 'de',
            },
            mode='display')
        self.checkOutput("""
        <div class="expiration-widget"><input disabled="disabled"
        id="checkbox-active" type="checkbox" /></div>
        """, widget())

        widget = factory(
            'expiration',
            name='active',
            value=1356217200.0,
            props={
                'datepicker': True,
                'locale': 'de',
                'format': '%Y.%m.%d',
            },
            mode='display')
        self.checkOutput("""
        <div class="expiration-widget"><input checked="checked"
        disabled="disabled" id="checkbox-active"
        type="checkbox" /><label>until</label><div class="display-expiration
        form-control" id="display-active">2012.12.23</div></div>
        """, widget())



import sys
import unittest


def test_suite():
    from cone.ugm.tests import test_layout
    from cone.ugm.tests import test_localmanager
    from cone.ugm.tests import test_settings
    from cone.ugm.tests import test_utils

    from cone.ugm.tests import test_model_group
    from cone.ugm.tests import test_model_groups
    from cone.ugm.tests import test_model_user
    from cone.ugm.tests import test_model_users

    from cone.ugm.tests import test_browser_actions
    from cone.ugm.tests import test_browser_autoincrement
    from cone.ugm.tests import test_browser_expires
    from cone.ugm.tests import test_browser_group
    from cone.ugm.tests import test_browser_groups
    from cone.ugm.tests import test_browser_portrait
    from cone.ugm.tests import test_browser_principal
    from cone.ugm.tests import test_browser_remote
    from cone.ugm.tests import test_browser_roles
    from cone.ugm.tests import test_browser_root
    from cone.ugm.tests import test_browser_settings
    from cone.ugm.tests import test_browser_user
    from cone.ugm.tests import test_browser_users
    from cone.ugm.tests import test_browser_utils

    suite = unittest.TestSuite()

    suite.addTest(unittest.findTestCases(test_layout))
    suite.addTest(unittest.findTestCases(test_localmanager))
    suite.addTest(unittest.findTestCases(test_settings))
    suite.addTest(unittest.findTestCases(test_utils))

    suite.addTest(unittest.findTestCases(test_model_group))
    suite.addTest(unittest.findTestCases(test_model_groups))
    suite.addTest(unittest.findTestCases(test_model_user))
    suite.addTest(unittest.findTestCases(test_model_users))

    suite.addTest(unittest.findTestCases(test_browser_actions))
    suite.addTest(unittest.findTestCases(test_browser_autoincrement))
    suite.addTest(unittest.findTestCases(test_browser_expires))
    suite.addTest(unittest.findTestCases(test_browser_group))
    suite.addTest(unittest.findTestCases(test_browser_groups))
    suite.addTest(unittest.findTestCases(test_browser_portrait))
    suite.addTest(unittest.findTestCases(test_browser_principal))
    suite.addTest(unittest.findTestCases(test_browser_remote))
    suite.addTest(unittest.findTestCases(test_browser_roles))
    suite.addTest(unittest.findTestCases(test_browser_root))
    suite.addTest(unittest.findTestCases(test_browser_settings))
    suite.addTest(unittest.findTestCases(test_browser_user))
    suite.addTest(unittest.findTestCases(test_browser_users))
    suite.addTest(unittest.findTestCases(test_browser_utils))

    return suite


def run_tests():
    from zope.testrunner.runner import Runner

    runner = Runner(found_suites=[test_suite()])
    runner.run()
    sys.exit(int(runner.failed))


if __name__ == '__main__':
    run_tests()



from cone.app import compat
from cone.app import get_root
from cone.tile.tests import TileTestCase
from cone.ugm import testing
from pyramid.exceptions import HTTPForbidden
from pyramid.view import render_view_to_response
import json


class TestBrowserActions(TileTestCase):
    layer = testing.ugm_layer

    @testing.principals(
        users={
            'user_1': {},
            'viewer': {},
            'editor': {}
        },
        groups={
            'group_1': {}
        },
        roles={
            'viewer': ['viewer'],
            'editor': ['editor']
        })
    def test_group_add_user_action(self):
        root = get_root()
        groups = root['groups']
        group = groups['group_1']

        request = self.layer.new_request(type='json')
        with self.layer.authenticated('viewer'):
            self.expectError(
                HTTPForbidden,
                render_view_to_response,
                group,
                request,
                name='add_item'
            )

        request.params['id'] = 'user_1'
        with self.layer.authenticated('editor'):
            res = render_view_to_response(group, request, name='add_item')
        self.assertEqual(json.loads(res.text), {
            'message': "Added user 'user_1' to group 'group_1'.",
            'success': True
        })

        request.params['id'] = 'user_2'
        with self.layer.authenticated('editor'):
            res = render_view_to_response(group, request, name='add_item')
        json_res = json.loads(res.text)
        self.assertFalse(json_res['success'])
        self.assertTrue(json_res['message'].find("'user_2'") > -1)

        self.assertEqual(group.model.member_ids, ['user_1'])

    @testing.principals(
        users={
            'user_1': {},
            'viewer': {},
            'editor': {}
        },
        groups={
            'group_1': {}
        },
        membership={
            'group_1': ['user_1']
        },
        roles={
            'viewer': ['viewer'],
            'editor': ['editor']
        })
    def test_group_remove_user_action(self):
        root = get_root()
        groups = root['groups']
        group = groups['group_1']

        request = self.layer.new_request(type='json')
        with self.layer.authenticated('viewer'):
            self.expectError(
                HTTPForbidden,
                render_view_to_response,
                group,
                request,
                name='remove_item'
            )

        request.params['id'] = 'user_2'
        with self.layer.authenticated('editor'):
            res = render_view_to_response(group, request, name='remove_item')
        json_res = json.loads(res.text)
        self.assertFalse(json_res['success'])
        self.assertTrue(json_res['message'].find("'user_2'") > -1)

        request.params['id'] = 'user_1'
        with self.layer.authenticated('editor'):
            res = render_view_to_response(group, request, name='remove_item')
        self.assertEqual(json.loads(res.text), {
            'message': "Removed user 'user_1' from group 'group_1'.",
            'success': True
        })

        self.assertEqual(group.model.users, [])

    @testing.principals(
        users={
            'user_1': {},
            'viewer': {},
            'editor': {}
        },
        groups={
            'group_1': {}
        },
        roles={
            'viewer': ['viewer'],
            'editor': ['editor']
        })
    def test_user_add_to_group_action(self):
        root = get_root()
        users = root['users']
        user = users['user_1']

        request = self.layer.new_request(type='json')
        with self.layer.authenticated('viewer'):
            self.expectError(
                HTTPForbidden,
                render_view_to_response,
                user,
                request,
                name='add_item'
            )

        request.params['id'] = 'group_1'
        with self.layer.authenticated('editor'):
            res = render_view_to_response(user, request, name='add_item')
        self.assertEqual(json.loads(res.text), {
            'message': "Added user 'user_1' to group 'group_1'.",
            'success': True
        })

        request.params['id'] = 'group_2'
        with self.layer.authenticated('editor'):
            res = render_view_to_response(user, request, name='add_item')
        json_res = json.loads(res.text)
        self.assertFalse(json_res['success'])
        self.assertTrue(json_res['message'].find("'group_2'") > -1)

        self.assertEqual(user.model.group_ids, ['group_1'])

    @testing.principals(
        users={
            'user_1': {},
            'viewer': {},
            'editor': {}
        },
        groups={
            'group_1': {}
        },
        membership={
            'group_1': ['user_1']
        },
        roles={
            'viewer': ['viewer'],
            'editor': ['editor']
        })
    def test_user_remove_from_group_action(self):
        root = get_root()
        users = root['users']
        user = users['user_1']

        request = self.layer.new_request(type='json')
        with self.layer.authenticated('viewer'):
            self.expectError(
                HTTPForbidden,
                render_view_to_response,
                user,
                request,
                name='remove_item'
            )

        request.params['id'] = 'group_2'
        with self.layer.authenticated('editor'):
            res = render_view_to_response(user, request, name='remove_item')
        json_res = json.loads(res.text)
        self.assertFalse(json_res['success'])
        self.assertTrue(json_res['message'].find("'group_2'") > -1)

        request.params['id'] = 'group_1'
        with self.layer.authenticated('editor'):
            res = render_view_to_response(user, request, name='remove_item')
        self.assertEqual(json.loads(res.text), {
            'message': "Removed user 'user_1' from group 'group_1'.",
            'success': True
        })

        self.assertEqual(user.model.groups, [])

    @testing.principals(
        users={
            'user_1': {},
            'viewer': {},
            'admin': {}
        },
        groups={
            'group_1': {}
        },
        membership={
            'group_1': ['user_1']
        },
        roles={
            'viewer': ['viewer'],
            'admin': ['admin']
        })
    def test_delete_user_action(self):
        root = get_root()
        groups = root['groups']
        users = root['users']

        group = groups['group_1']
        user = users['user_1']

        request = self.layer.new_request(type='json')
        with self.layer.authenticated('viewer'):
            self.expectError(
                HTTPForbidden,
                render_view_to_response,
                user,
                request,
                name='delete_item'
            )

        with self.layer.authenticated('admin'):
            res = render_view_to_response(user, request, name='delete_item')
        self.assertEqual(json.loads(res.text), {
            'message': "Deleted user 'user_1' from database.",
            'success': True
        })

        with self.layer.authenticated('admin'):
            res = render_view_to_response(user, request, name='delete_item')
        json_res = json.loads(res.text)
        self.assertFalse(json_res['success'])
        self.assertTrue(json_res['message'].find("'user_1'") > -1)

        self.assertEqual(group.model.users, [])

    @testing.principals(
        users={
            'viewer': {},
            'admin': {}
        },
        groups={
            'group_1': {}
        },
        roles={
            'viewer': ['viewer'],
            'admin': ['admin']
        })
    def test_delete_group_action(self):
        root = get_root()
        groups = root['groups']
        group = groups['group_1']

        request = self.layer.new_request(type='json')
        with self.layer.authenticated('viewer'):
            self.expectError(
                HTTPForbidden,
                render_view_to_response,
                group,
                request,
                name='delete_item'
            )

        with self.layer.authenticated('admin'):
            res = render_view_to_response(group, request, name='delete_item')
        self.assertEqual(json.loads(res.text), {
            'message': "Deleted group from database",
            'success': True
        })

        with self.layer.authenticated('admin'):
            res = render_view_to_response(group, request, name='delete_item')
        json_res = json.loads(res.text)
        self.assertFalse(json_res['success'])
        self.assertTrue(json_res['message'].find("'group_1'") > -1)

        self.assertEqual(groups.keys(), [])



from cone.app import root
from cone.app.model import Metadata
from cone.app.model import Properties
from cone.ugm import testing
from cone.ugm import ugm_default_acl
from cone.ugm.model.user import User
from cone.ugm.model.users import Users
from node.ext.ugm.interfaces import IUsers
from node.tests import NodeTestCase


class TestModelUsers(NodeTestCase):
    layer = testing.ugm_layer

    @testing.principals(
        users={
            'user_1': {},
            'user_2': {},
        })
    def test_users(self):
        # Users container
        users = root['users']
        self.assertTrue(isinstance(users, Users))
        self.assertEqual(users.name, 'users')

        # Properties
        self.assertTrue(isinstance(users.properties, Properties))

        # Metadata
        md = users.metadata
        self.assertTrue(isinstance(md, Metadata))
        self.assertEqual(md.title, 'users_node')
        self.assertEqual(md.description, 'users_node_description')

        # Iter users
        self.assertEqual(len([x for x in users]), 2)

        # Inexistent child
        self.expect_error(KeyError, users.__getitem__, 'inexistent')

        # Children are user application nodes
        user = users['user_1']
        self.assertTrue(isinstance(user, User))

        # Check UGM backend
        backend = users.backend
        self.assertTrue(IUsers.providedBy(backend))

        # Check invalidate
        self.assertTrue(backend is users.backend)
        users.invalidate()
        self.assertFalse(backend is users.backend)

        # ACL
        self.assertEqual(users.__acl__, ugm_default_acl)



from cone.app import root
from cone.app.ugm import ugm_backend
from cone.ugm import testing
from cone.ugm.settings import GeneralSettings
from cone.ugm.utils import general_settings
from node.ext.ugm.interfaces import IUgm
import unittest


class TestModelUtils(unittest.TestCase):
    layer = testing.ugm_layer

    def test_general_settings(self):
        settings = general_settings(root)
        self.assertTrue(isinstance(settings, GeneralSettings))
        self.assertEqual(settings.name, 'ugm_general')

    def test_ugm_backend(self):
        backend = ugm_backend.ugm
        self.assertTrue(IUgm.providedBy(backend))

        self.assertTrue(backend is ugm_backend.ugm)
        ugm_backend.initialize()
        self.assertFalse(backend is ugm_backend.ugm)



from cone.app import compat
from cone.app.browser.batch import Batch
from cone.app.browser.utils import make_query
from cone.app.browser.utils import make_url
from cone.app.browser.utils import nodepath
from cone.app.browser.utils import request_property
from cone.app.browser.utils import safe_decode
from cone.tile import Tile
from cone.ugm.utils import general_settings
from pyramid.i18n import TranslationStringFactory
from yafowil.utils import Tag
import logging
import natsort


tag = Tag(lambda x: x)


logger = logging.getLogger('cone.ugm')
_ = TranslationStringFactory('cone.ugm')


class ColumnListingBatch(Batch):
    """Column listing batch.
    """

    def __init__(self, listing):
        self.listing = listing
        self.name = listing.batchname
        self.items = listing.listing_items
        self.slicesize = listing.slicesize

    @property
    def display(self):
        return len(self.vocab) > 1

    @property
    def vocab(self):
        ret = list()
        path = nodepath(self.model)
        count = len(self.items)
        pages = count // self.slicesize
        if count % self.slicesize != 0:
            pages += 1
        current = self.listing.current_page
        for i in range(pages):
            ret.append({
                'page': '%i' % (i + 1),
                'current': current == i,
                'visible': True,
                'target': self.listing.batch_target(path, i)
            })
        return ret


class ColumnListing(Tile):
    """Abstract column listing.
    """
    current_id = None
    slot = None
    list_columns = []
    css = ''
    slicesize = 8
    batchname = ''
    default_order = 'asc'
    display_filter = True
    display_limit = False
    display_limit_checked = False

    @property
    def ajax_action(self):
        return 'columnlisting'

    @property
    def ajax_event(self):
        return '{}:{}'.format(
            self.batch.trigger_event,
            self.batch.trigger_selector
        )

    @property
    def sortheader(self):
        ret = list()
        for cid, name in self.list_columns.items():
            cur_sort = self.sort_column
            cur_order = self.sort_order
            selected = cur_sort == cid
            alter = selected and cur_order == 'asc'
            order = alter and 'desc' or 'asc'
            ret.append({
                'title': name,
                'order': order if selected else '',
                'target': self.sort_target(cid, order)
            })
        return ret

    def unquoted_param_value(self, name):
        value = self.request.params.get(name)
        if value:
            value = value.encode('utf-8') if compat.IS_PY2 else value
            value = compat.unquote(value)
            value = value.decode('utf-8') if compat.IS_PY2 else value
        return value

    def filter_value_or_default(self, name):
        value = self.unquoted_param_value(name)
        return value if value else _('filter_listing', default='filter listing')

    @property
    def filter_target(self):
        query = make_query(dict(sort=self.sort_column, order=self.sort_order))
        return safe_decode(make_url(self.request, node=self.model, query=query))

    @property
    def filter_term(self):
        return self.unquoted_param_value('filter')

    @property
    def filter_value(self):
        return self.filter_value_or_default('filter')

    @property
    def default_sort(self):
        return self.list_columns.keys()[0]

    @property
    def sort_column(self):
        return self.request.params.get('sort', self.default_sort)

    @property
    def sort_order(self):
        return self.request.params.get('order', self.default_order)

    def sort_target(self, sort, order):
        query = make_query(
            filter=self.filter_term,
            b_page=self.current_page,
            sort=sort,
            order=order)
        return safe_decode(make_url(self.request, node=self.model, query=query))

    @property
    def current_page(self):
        return int(self.request.params.get('b_page', '0'))

    @request_property
    def batch(self):
        return ColumnListingBatch(self)

    @property
    def rendered_batch(self):
        return self.batch(self.model, self.request)

    def batch_target(self, path, b_page):
        query = make_query(
            b_page=str(b_page),
            filter=self.filter_term,
            sort=self.sort_column,
            order=self.sort_order)
        return safe_decode(make_url(self.request, path=path, query=query))

    @property
    def slice(self):
        start = self.current_page * self.slicesize
        end = start + self.slicesize
        return start, end

    @property
    def items(self):
        start, end = self.slice
        items = self.listing_items
        inv = self.sort_order == 'desc'
        items = natsort.natsorted(
            items,
            key=lambda x: x['sort_by'],
            reverse=inv,
            alg=natsort.ns.IC
        )
        return items[start:end]

    @property
    def listing_items(self):
        """Return list of dicts like:

        {
            'target': u'http://example.com/foo',
            'content': u'Head Column content',
            'sort_by: u'Sort value',
            'current': False,
            'actions': [
                {
                    'id': 'action_id',
                    'enabled': True,
                    'title': 'Action Title',
                    'target': u'http://example.com/foo',
                }
            ],
        }
        """
        raise NotImplementedError(
            'Abstract ``ColumnListing`` does not implement ``listing_items``')

    def item_content(self, *args):
        ret = u''
        pt = 0
        for arg in args:
            pt += 1
            ret += tag('div', '%s&nbsp;' % arg, class_='sort_col_%i' % pt)
        return ret

    def create_item(self, sort, target, content, current, actions):
        return {
            'sort_by': sort,
            'target': target,
            'content': content,
            'current': current,
            'actions': actions,
        }

    def create_action(self, aid, enabled, title, target):
        return {
            'id': aid,
            'enabled': enabled,
            'title': title,
            'target': target,
        }

    def extract_raw(self, attrs, name):
        raw = attrs.get(name)
        if type(raw) in compat.ITER_TYPES:
            return raw[0]
        return raw and raw or ''

    ############################################################
    # XXX: users and groups related info to dedicated subclasses

    # USERS RELATED

    @property
    def user_list_columns(self):
        settings = general_settings(self.model)
        return settings.attrs.users_listing_columns

    @property
    def user_attrs(self):
        settings = general_settings(self.model)
        return settings.attrs.users_listing_columns.keys()

    @property
    def user_localmanager_ids(self):
        if not self.model.local_manager_consider_for_user:
            return None
        return self.model.local_manager_target_uids

    @property
    def user_default_sort_column(self):
        settings = general_settings(self.model)
        attrs = self.user_attrs
        sort = settings.attrs.users_listing_default_column
        if sort not in attrs:
            return attrs[0]
        return sort

    # GROUPS RELATED

    @property
    def group_list_columns(self):
        settings = general_settings(self.model)
        return settings.attrs.groups_listing_columns

    @property
    def group_attrs(self):
        settings = general_settings(self.model)
        return settings.attrs.groups_listing_columns.keys()

    @property
    def group_localmanager_ids(self):
        if not self.model.local_manager_consider_for_user:
            return None
        return self.model.local_manager_target_gids

    @property
    def group_default_sort_column(self):
        settings = general_settings(self.model)
        attrs = self.group_attrs
        sort = settings.attrs.groups_listing_default_column
        if sort not in attrs:
            return attrs[0]
        return sort

    # XXX: end move
    ############################################################


class PrincipalsListing(ColumnListing):
    """Column listing for principals.
    """
    delete_label = _('delete_principal', default='Delete Principal')
    delete_permission = 'delete_principal'  # inexistent permission
    listing_attrs = []
    localmanager_ids = None
    sort_attr = None

    @request_property
    def listing_items(self):
        can_delete = self.request.has_permission(
            self.delete_permission,
            self.model
        )
        try:
            ret = list()
            localmanager_ids = self.localmanager_ids
            # if localmanager ids not none but empty, no access to any
            # principals
            if localmanager_ids is not None and not localmanager_ids:
                return ret
            attrlist = self.listing_attrs
            sort_attr = self.sort_attr
            # build criteria from filter term
            criteria = None
            filter_term = self.filter_term
            if filter_term:
                criteria = dict()
                for attr in attrlist:
                    criteria[attr] = filter_term
            principals = self.model.backend
            result = principals.search(
                criteria=criteria,
                attrlist=attrlist,
                or_search=True
            )
            for key, attrs in result:
                # reduce result by localmanager ids if not None
                if localmanager_ids is not None and key not in localmanager_ids:
                    continue
                query = make_query(
                    pid=key,
                    came_from=make_url(self.request, node=self.model)
                )
                target = make_url(self.request, node=self.model, query=query)
                actions = list()
                if can_delete:
                    action_id = 'delete_item'
                    action_title = self.delete_label
                    delete_action = self.create_action(
                        action_id,
                        True,
                        action_title,
                        target
                    )
                    actions = [delete_action]
                vals = [self.extract_raw(attrs, attr) for attr in attrlist]
                sort = self.extract_raw(attrs, sort_attr)
                content = self.item_content(*vals)
                current = self.current_id == key
                ret.append(self.create_item(
                    sort,
                    target,
                    content,
                    current,
                    actions
                ))
            return ret
        except Exception:
            logger.exception('Failed to query listing items')
        return list()



from cone.app import get_root
from cone.app.model import BaseNode
from cone.ugm import testing
from cone.ugm.localmanager import LocalManager
from cone.ugm.localmanager import LocalManagerConfigAttributes
from node.tests import NodeTestCase
from plumber import plumbing
import os


class TestModelLocalmanager(NodeTestCase):
    layer = testing.ugm_layer

    @testing.temp_directory
    def test_LocalManagerConfigAttributes(self, tempdir):
        # Local manager configuration attributes
        conf_path = os.path.join(tempdir, 'localmanager.xml')
        config = LocalManagerConfigAttributes(conf_path)

        # Not exists yet
        self.assertFalse(os.path.exists(conf_path))

        # After calling it does
        config()
        self.assertTrue(os.path.exists(conf_path))

        with open(conf_path) as handle:
            data = handle.read().split('\n')
        self.assertEqual(data, ['<localmanager/>', ''])

        # Add rules
        config['foo'] = {
            'target': ['bar', 'baz'],
            'default': ['bar'],
        }
        config['aaa'] = {
            'target': ['bbb', 'ccc'],
            'default': ['ccc'],
        }

        # Iter
        self.assertEqual(sorted(list(config)), ['aaa', 'foo'])

        # Modify a rule
        config['foo']['default'] = ['bar']

        # Write config to file
        config()

        # Recreate on existing conf
        config = LocalManagerConfigAttributes(conf_path)
        self.assertEqual(sorted(config.items()), [
            ('aaa', {'default': ['ccc'], 'target': ['bbb', 'ccc']}),
            ('foo', {'default': ['bar'], 'target': ['bar', 'baz']})
        ])

    @testing.principals(
        users={
            'local_manager_1': {},
            'local_manager_2': {},
            'managed_user_1': {},
            'managed_user_2': {}
        },
        groups={
            'admin_group_1': {},
            'admin_group_2': {},
            'managed_group_0': {},
            'managed_group_1': {},
            'managed_group_2': {}
        },
        membership={
            'admin_group_1': ['local_manager_1'],
            'admin_group_2': ['local_manager_2'],
            'managed_group_1': ['managed_user_1'],
            'managed_group_2': ['managed_user_1', 'managed_user_2'],
        })
    def test_LocalManager(self):
        root = get_root()
        config = root['settings']['ugm_localmanager'].attrs
        self.assertEqual(sorted(config.items()), [
            ('admin_group_1', {
                'default': ['managed_group_1'],
                'target': ['managed_group_0', 'managed_group_1']
            }),
            ('admin_group_2', {
                'default': ['managed_group_2'],
                'target': ['managed_group_1', 'managed_group_2']
            })
        ])

        # Local Manager plumbing behavior
        @plumbing(LocalManager)
        class LocalManagerNode(BaseNode):
            pass

        lm_node = LocalManagerNode(name='lm_node', parent=root)
        self.assertFalse(lm_node.local_management_enabled)

        # ``local_management_enabled`` is generally ignored in following
        # functions of ``LocalManager``. User needs to consider if local
        # management is enabled.

        # Unauthenticated
        self.layer.new_request()
        self.assertEqual(lm_node.local_manager_target_gids, [])
        self.assertEqual(lm_node.local_manager_target_uids, [])

        # Authenticated, no local manager
        with self.layer.authenticated('inexistent'):
            self.assertEqual(lm_node.local_manager_target_gids, [])
            self.assertEqual(lm_node.local_manager_target_uids, [])

        # Authenticated, invalid local management group member
        groups = root['groups'].backend
        group = groups['admin_group_2']
        group.add('local_manager_1')
        group()
        self.assertEqual(sorted(group.member_ids), [
            'local_manager_1',
            'local_manager_2'
        ])

        with self.layer.authenticated('local_manager_1'):
            err = self.expectError(
                Exception,
                lambda: lm_node.local_manager_target_gids
            )
        expected = (
            "Authenticated member defined in local manager groups "
            "'admin_group_1', 'admin_group_2' but only one management group allowed "
            "for each user. Please contact System Administrator in order to fix "
            "this problem."
        )
        self.assertEqual(str(err), expected)

        del group['local_manager_1']
        group()
        self.assertEqual(group.member_ids, [u'local_manager_2'])

        # Authenticated, local manager
        with self.layer.authenticated('local_manager_1'):
            self.assertEqual(
                sorted(lm_node.local_manager_target_gids),
                ['managed_group_0', 'managed_group_1']
            )
            self.assertEqual(
                lm_node.local_manager_target_uids,
                ['managed_user_1']
            )

        with self.layer.authenticated('local_manager_2'):
            self.assertEqual(
                sorted(lm_node.local_manager_target_gids),
                ['managed_group_1', 'managed_group_2']
            )
            self.assertEqual(
                sorted(lm_node.local_manager_target_uids),
                ['managed_user_1', 'managed_user_2']
            )

        # Check if group id is marked as default
        self.assertFalse(lm_node.local_manager_is_default(
            'admin_group_1',
            'managed_group_0'
        ))
        err = self.expect_error(
            Exception,
            lm_node.local_manager_is_default,
            'admin_group_2',
            'managed_group_0'
        )
        expected = "group 'managed_group_0' not managed by 'admin_group_2'"
        self.assertEqual(str(err), expected)
        self.assertTrue(lm_node.local_manager_is_default(
            'admin_group_1',
            'managed_group_1'
        ))
        self.assertFalse(lm_node.local_manager_is_default(
            'admin_group_2',
            'managed_group_1'
        ))
        err = self.expect_error(
            Exception,
            lm_node.local_manager_is_default,
            'admin_group_1',
            'managed_group_2'
        )
        expected = "group 'managed_group_2' not managed by 'admin_group_1'"
        self.assertEqual(str(err), expected)
        self.assertTrue(lm_node.local_manager_is_default(
            'admin_group_2',
            'managed_group_2'
        ))

    @testing.principals(
        users={
            'local_manager_1': {},
            'local_manager_2': {},
            'managed_user_1': {},
            'managed_user_2': {}
        },
        groups={
            'admin_group_1': {},
            'admin_group_2': {},
            'managed_group_0': {},
            'managed_group_1': {},
            'managed_group_2': {}
        },
        membership={
            'admin_group_1': ['local_manager_1'],
            'admin_group_2': ['local_manager_2'],
            'managed_group_1': ['managed_user_1'],
            'managed_group_2': ['managed_user_1', 'managed_user_2'],
        })
    def test_LocalManagerACL(self):
        root = get_root()
        self.layer.new_request()

        # Local manager ACL for users node
        users = root['users']
        self.assertEqual(users.local_manager_acl, [])

        with self.layer.authenticated('managed_user_1'):
            self.assertEqual(users.local_manager_acl, [])

        with self.layer.authenticated('local_manager_1'):
            self.assertEqual(users.local_manager_acl, [
                ('Allow', 'local_manager_1', [
                    'view', 'add', 'add_user', 'edit', 'edit_user',
                    'manage_expiration', 'manage_membership'
                ])
            ])

        # Local manager ACL for groups node
        groups = root['groups']
        self.assertEqual(groups.local_manager_acl, [])

        with self.layer.authenticated('managed_user_1'):
            self.assertEqual(groups.local_manager_acl, [])

        with self.layer.authenticated('local_manager_1'):
            self.assertEqual(groups.local_manager_acl, [
                ('Allow', 'local_manager_1', ['view', 'manage_membership'])
            ])

        # Local manager ACL for group node
        managed_group_0 = groups['managed_group_0']
        managed_group_1 = groups['managed_group_1']
        managed_group_2 = groups['managed_group_2']

        self.assertEqual(managed_group_0.local_manager_acl, [])
        self.assertEqual(managed_group_1.local_manager_acl, [])
        self.assertEqual(managed_group_2.local_manager_acl, [])

        with self.layer.authenticated('managed_user_1'):
            self.assertEqual(managed_group_0.local_manager_acl, [])
            self.assertEqual(managed_group_1.local_manager_acl, [])
            self.assertEqual(managed_group_2.local_manager_acl, [])

        with self.layer.authenticated('local_manager_1'):
            self.assertEqual(managed_group_0.local_manager_acl, [
                ('Allow', 'local_manager_1', ['view', 'manage_membership'])
            ])
            self.assertEqual(managed_group_1.local_manager_acl, [
                ('Allow', 'local_manager_1', ['view', 'manage_membership'])
            ])
            self.assertEqual(managed_group_2.local_manager_acl, [])

        with self.layer.authenticated('local_manager_2'):
            self.assertEqual(managed_group_0.local_manager_acl, [])
            self.assertEqual(managed_group_1.local_manager_acl, [
                ('Allow', 'local_manager_2', ['view', 'manage_membership'])
            ])
            self.assertEqual(managed_group_2.local_manager_acl, [
                ('Allow', 'local_manager_2', ['view', 'manage_membership'])
            ])

        # Local manager ACL for user node
        managed_user_1 = users['managed_user_1']
        managed_user_2 = users['managed_user_2']

        self.assertEqual(managed_user_1.local_manager_acl, [])
        self.assertEqual(managed_user_2.local_manager_acl, [])

        with self.layer.authenticated('managed_user_1'):
            self.assertEqual(managed_user_1.local_manager_acl, [])
            self.assertEqual(managed_user_2.local_manager_acl, [])

        with self.layer.authenticated('local_manager_1'):
            self.assertEqual(managed_user_1.local_manager_acl, [
                ('Allow', 'local_manager_1', [
                    'view', 'add', 'add_user', 'edit', 'edit_user',
                    'manage_expiration', 'manage_membership'
                ])
            ])
            self.assertEqual(managed_user_2.local_manager_acl, [])

        with self.layer.authenticated('local_manager_2'):
            self.assertEqual(managed_user_1.local_manager_acl, [
                ('Allow', 'local_manager_2', [
                    'view', 'add', 'add_user', 'edit', 'edit_user',
                    'manage_expiration', 'manage_membership'
                ])
            ])
            self.assertEqual(managed_user_2.local_manager_acl, [
                ('Allow', 'local_manager_2', [
                    'view', 'add', 'add_user', 'edit', 'edit_user',
                    'manage_expiration', 'manage_membership'
                ])
            ])



from cone.app.model import BaseNode
from cone.app.model import Metadata
from cone.app.model import Properties
from cone.app.model import XMLProperties
from cone.ugm.localmanager import LocalManagerConfigAttributes
from cone.ugm.utils import general_settings
from node.utils import instance_property
from pyramid.i18n import TranslationStringFactory
import os


_ = TranslationStringFactory('cone.ugm')


ugm_cfg = Properties()
ugm_cfg.ugm_settings = ''
ugm_cfg.lm_settings = ''

# XXX: move cone.ugm.model.factory_defaults here


class UGMSettings(BaseNode):
    config_file = None

    def __call__(self):
        self.attrs()

    @instance_property
    def attrs(self):
        config_file = self.config_file
        if not os.path.isfile(config_file):
            msg = 'Configuration file {} not exists.'.format(config_file)
            raise ValueError(msg)
        return XMLProperties(config_file)

    def invalidate(self, attrs=[]):
        attrs.append('attrs')
        for attr in attrs:
            _attr = '_{}'.format(attr)
            if hasattr(self, _attr):
                delattr(self, _attr)


class GeneralSettings(UGMSettings):

    @property
    def config_file(self):
        return ugm_cfg.ugm_settings

    @instance_property
    def metadata(self):
        metadata = Metadata()
        metadata.title = _(
            'ugm_settings_node',
            default='UGM Settings')
        metadata.description = _(
            'ugm_settings_node_description',
            default='General user and group management settings'
        )
        return metadata


class LocalManagerSettings(BaseNode):

    def attributes_factory(self, name=None, parent=None):
        return LocalManagerConfigAttributes(ugm_cfg.lm_settings)

    @instance_property
    def metadata(self):
        metadata = Metadata()
        metadata.title = _(
            'localmanager_settings_node',
            default='Local Manager Settings'
        )
        metadata.description = _(
            'localmanager_settings_node_description',
            default='Local Manager Settings'
        )
        return metadata

    @property
    def enabled(self):
        settings = general_settings(self.root)
        return settings.attrs.users_local_management_enabled == 'True'

    def __call__(self):
        self.attrs()



from cone.app.browser.form import Form
from cone.app.browser.form import YAMLForm
from cone.app.browser.layout import ProtectedContentTile
from cone.app.browser.settings import SettingsBehavior
from cone.app.ugm import ugm_backend
from cone.tile import tile
from cone.ugm.settings import GeneralSettings
from cone.ugm.settings import LocalManagerSettings
from plumber import plumbing
from pyramid.i18n import TranslationStringFactory
from pyramid.view import view_config
from yafowil.base import ExtractionError
from yafowil.base import UNSET


_ = TranslationStringFactory('cone.ugm')


@tile(
    name='content',
    path='templates/general_settings.pt',
    interface=GeneralSettings,
    permission='manage')
class GeneralSettingsContent(ProtectedContentTile):
    pass


@tile(name='editform', interface=GeneralSettings, permission='manage')
@plumbing(SettingsBehavior, YAMLForm)
class GeneralSettingsForm(Form):
    action_resource = u'edit'
    form_template = 'cone.ugm.browser:forms/general_settings.yaml'

    @property
    def message_factory(self):
        return _

    def required_if_users_account_expiration(self, widget, data):
        extracted = data.extracted
        if extracted is UNSET:
            return extracted
        if data.root['users_account_expiration'].extracted and not extracted:
            raise ExtractionError(_(
                'required_if_users_account_expiration',
                default='Value is required if account expiration is enabled'
            ))
        return extracted

    def required_if_users_portrait(self, widget, data):
        extracted = data.extracted
        if extracted is UNSET:
            return extracted
        if data.root['users_portrait'].extracted and not extracted:
            raise ExtractionError(_(
                'required_if_users_portrait',
                default='Value is required if portrit support is enabled'
            ))
        return extracted

    def save(self, widget, data):
        # XXX: user data.write(model)
        model = self.model
        for attr_name in [
            'users_account_expiration',
            'users_expires_attr',
            'users_expires_unit',
            'user_id_autoincrement',
            'user_id_autoincrement_prefix',
            'user_id_autoincrement_start',
            'users_portrait',
            'users_portrait_attr',
            'users_portrait_accept',
            'users_portrait_width',
            'users_portrait_height',
            'users_local_management_enabled',
            'users_login_name_attr',
            'users_exposed_attributes',
            'users_form_attrmap',
            'users_listing_columns',
            'users_listing_default_column',
            'groups_form_attrmap',
            'groups_listing_columns',
            'groups_listing_default_column',
            'roles_principal_roles_enabled'
        ]:
            val = data.fetch('ugm_settings.%s' % attr_name).extracted
            setattr(model.attrs, attr_name, val)
        model()
        model.invalidate()
        ugm_backend.initialize()


@tile(
    name='content',
    path='templates/localmanager_settings.pt',
    interface=LocalManagerSettings,
    permission='manage')
class LocalManagerSettingsTile(ProtectedContentTile):
    pass


@tile(name='editform', interface=LocalManagerSettings, permission='manage')
@plumbing(SettingsBehavior, YAMLForm)
class LocalManagerSettingsForm(Form):
    action_resource = u'edit'
    form_template = 'cone.ugm.browser:forms/localmanager_settings.yaml'

    @property
    def message_factory(self):
        return _

    @property
    def rules_value(self):
        """Return value format:

            return [{
                'source': 'aaa',
                'targets': [{
                    'gid': 'bbb',
                    'default': False,
                }]
            }]
        """
        rules = list()
        items = self.model.attrs.items()
        items = sorted(items, key=lambda x: x[0])
        for source, defs in items:
            rule = dict()
            rule['source'] = source
            rule['targets'] = list()
            targets = sorted(defs['target'])
            for gid in targets:
                rule['targets'].append({
                    'gid': gid,
                    'default': gid in defs['default']
                })
            rules.append(rule)
        return rules

    def duplicate_rule(self, widget, data):
        """Check for duplicate rules.
        """
        source = data.extracted['source']
        if not source:
            return data.extracted
        exists = [source]
        for val in data.parent.values():
            if val.name == data.name:
                continue
            other = val.extracted['source']
            if other in exists:
                raise ExtractionError(_(
                    'localmanager_duplicate_rule_error',
                    default='Duplicate access rule'
                ))
            exists.append(other)
        return data.extracted

    def target_not_source(self, widget, data):
        """Check whether source and target are same.
        """
        source = data.parent.parent.parent['source'].extracted
        if source == data.extracted:
            raise ExtractionError(_(
                'localmanager_target_is_source_error',
                default='Target GID equates source GID'
            ))
        return data.extracted

    def save(self, widget, data):
        """save rules.
        """
        attrs = self.model.attrs
        recent = attrs.keys()
        extracted = data.fetch('localmanager_settings.rules').extracted
        for entry in extracted:
            source = entry['source']
            if source in recent:
                recent.remove(source)
            targets = set()
            defaults = set()
            for target in entry['targets']:
                targets.add(target['gid'])
                if target['default']:
                    defaults.add(target['gid'])
            rule = {
                'target': list(targets),
                'default': list(defaults),
            }
            attrs[source] = rule
        for source in recent:
            del attrs[source]
        self.model()


@view_config(
    name='group_id_vocab',
    accept='application/json',
    renderer='json',
    permission='manage')
def group_id_vocab(model, request):
    term = request.params['term']
    if len(term) < 2:
        return []
    backend = model.root['groups'].backend
    return backend.search(criteria={'id': '%s*' % term})



from cone.app import root
from cone.tile import render_tile
from cone.tile.tests import TileTestCase
from cone.ugm import testing
from pyramid.httpexceptions import HTTPForbidden


class TestBrowserUsers(TileTestCase):
    layer = testing.ugm_layer

    def test_content_tile(self):
        users = root['users']
        request = self.layer.new_request()

        # Unauthenticated content tile renders login form
        expected = '<form action="http://example.com/users/login"'
        res = render_tile(users, request, 'content')
        self.assertTrue(res.find(expected) > -1)

    @testing.principals(
        users={
            'manager': {},
        },
        roles={
            'manager': ['manager']
        })
    def test_leftcolumn_tile(self):
        users = root['users']
        request = self.layer.new_request()

        self.expectError(
            HTTPForbidden,
            render_tile,
            users,
            request,
            'leftcolumn'
        )

        with self.layer.authenticated('manager'):
            res = render_tile(users, request, 'leftcolumn')
        expected = '<div class="column left_column col-md-6">'
        self.assertTrue(res.find(expected) > -1)

    @testing.principals(
        users={
            'manager': {},
        },
        roles={
            'manager': ['manager']
        })
    def test_rightcolumn_tile(self):
        users = root['users']
        request = self.layer.new_request()

        self.expectError(
            HTTPForbidden,
            render_tile,
            users,
            request,
            'rightcolumn'
        )

        with self.layer.authenticated('manager'):
            res = render_tile(users, request, 'rightcolumn')
        expected = '<div class="column right_column col-md-6">'
        self.assertTrue(res.find(expected) > -1)

    @testing.principals(
        users={
            'manager': {},
        },
        roles={
            'manager': ['manager']
        })
    def test_columnlisting_tile(self):
        users = root['users']
        request = self.layer.new_request()

        self.expectError(
            HTTPForbidden,
            render_tile,
            users,
            request,
            'columnlisting'
        )

        with self.layer.authenticated('manager'):
            res = render_tile(users, request, 'columnlisting')
        expected = '<div class="columnlisting leftbatchsensitiv"'
        self.assertTrue(res.find(expected) > -1)



from cone.app import get_root
from cone.app.testing import Security
from cone.app.ugm import ugm_backend
from cone.ugm.settings import ugm_cfg
import os
import shutil
import tempfile


base_path = os.path.split(__file__)[0]
ugm_config = os.path.join(base_path, 'ugm.xml')
localmanager_config = os.path.join(base_path, 'localmanager.xml')


class principals(object):

    def __init__(self, users={}, groups={}, membership={}, roles={}):
        self.users = users
        self.groups = groups
        self.membership = membership
        self.roles = roles

    def apply(self):
        ugm_backend.ugm()
        root = get_root()
        root['users'].invalidate()
        root['groups'].invalidate()

    def create_principals(self):
        ugm = ugm_backend.ugm
        ugm_users = ugm.users
        ugm_groups = ugm.groups
        for user_id, user_kw in self.users.items():
            ugm_users.create(user_id, **user_kw)
            ugm_users()
            ugm_users.passwd(user_id, None, 'secret')
        for group_id, group_kw in self.groups.items():
            ugm_groups.create(group_id, **group_kw)
        for group_id, user_ids in self.membership.items():
            ugm_group = ugm_groups[group_id]
            for user_id in user_ids:
                ugm_group.add(user_id)
        for principal_id, roles in self.roles.items():
            if principal_id.startswith('group:'):
                principal = ugm_groups[principal_id[5:]]
            else:
                principal = ugm_users[principal_id]
            for role in roles:
                principal.add_role(role)
        self.apply()

    def remove_principals(self):
        ugm = ugm_backend.ugm
        ugm_users = ugm.users
        ugm_groups = ugm.groups
        for user_id in ugm_users.keys():
            try:
                del ugm_users[user_id]
                ugm_users()
            except KeyError:
                continue
            except Exception as e:
                print((
                    'Error while removing user. Please '
                    'check underlying UGM implementation: {}'
                ).format(e))
        for group_id in ugm_groups.keys():
            try:
                del ugm_groups[group_id]
                ugm_groups()
            except KeyError:
                continue
            except Exception as e:
                print((
                    'Error while removing group. Please '
                    'check underlying UGM implementation: {}'
                ).format(e))
        self.apply()

    def __call__(self, fn):
        def wrapper(inst):
            self.create_principals()
            try:
                fn(inst)
            finally:
                try:
                    self.remove_principals()
                except Exception as e:
                    raise e
        return wrapper


def _invalidate_settings():
    settings = get_root()['settings']
    settings['ugm_general'].invalidate()


def invalidate_settings(fn):
    """Decorator for tests working on settings nodes.
    """
    def wrapper(*a, **kw):
        _invalidate_settings()
        try:
            fn(*a, **kw)
        finally:
            _invalidate_settings()
    return wrapper


def custom_config_path(fn):
    """Decorator for tests writing to config files.
    """
    def wrapper(*a, **kw):
        ugm_settings = ugm_cfg.ugm_settings
        _invalidate_settings()
        try:
            fn(*a, **kw)
        finally:
            ugm_cfg.ugm_settings = ugm_settings
            _invalidate_settings()
    return wrapper


def temp_directory(fn):
    """Decorator for tests needing a temporary directory.
    """
    def wrapper(*a, **kw):
        tempdir = tempfile.mkdtemp()
        kw['tempdir'] = tempdir
        try:
            fn(*a, **kw)
        finally:
            shutil.rmtree(tempdir)
    return wrapper


class UGMLayer(Security):

    def make_app(self):
        ugm_users_file = os.path.join(self.ugm_dir, 'users')
        ugm_groups_file = os.path.join(self.ugm_dir, 'groups')
        ugm_roles_file = os.path.join(self.ugm_dir, 'roles')
        ugm_datadir = os.path.join(self.ugm_dir, 'data')
        super(UGMLayer, self).make_app(**{
            'cone.plugins': '\n'.join([
                'cone.ugm'
            ]),
            'ugm.backend': 'file',
            'ugm.config': ugm_config,
            'ugm.localmanager_config': localmanager_config,
            'ugm.users_file': ugm_users_file,
            'ugm.groups_file': ugm_groups_file,
            'ugm.roles_file': ugm_roles_file,
            'ugm.datadir': ugm_datadir
        })
        ugm_backend.initialize()

    def setUp(self, args=None):
        self.ugm_dir = tempfile.mkdtemp()
        super(UGMLayer, self).setUp(args=args)

    def tearDown(self):
        shutil.rmtree(self.ugm_dir)
        super(UGMLayer, self).tearDown()


ugm_layer = UGMLayer()



from cone.app.model import AdapterNode
from cone.app.model import Metadata
from cone.app.model import node_info
from cone.app.model import Properties
from cone.ugm.localmanager import LocalManagerUserACL
from node.locking import locktree
from node.utils import instance_property
from plumber import plumbing
from pyramid.i18n import TranslationStringFactory


_ = TranslationStringFactory('cone.ugm')


@node_info(
    'user',
    title=_('user_node', default='User'),
    description=_('user_node_description', default='User'))
@plumbing(LocalManagerUserACL)
class User(AdapterNode):

    @instance_property
    def properties(self):
        props = Properties()
        return props

    @instance_property
    def metadata(self):
        metadata = Metadata()
        metadata.title = _('user_node', default='User')
        metadata.description = _('user_node_description', default='User')
        return metadata

    @locktree
    def __call__(self):
        self.model()



from cone.app.browser.ajax import ajax_form_fiddle
from plumber import Behavior
from plumber import plumb


class AddFormFiddle(Behavior):
    """Form fiddle plumbing behavior for user and group add forms.
    """

    @plumb
    def __call__(_next, self, model, request):
        ajax_form_fiddle(request, '.right_column', 'inner')
        return _next(self, model, request)


class EditFormFiddle(Behavior):
    """Form fiddle plumbing behavior for user and group edit forms.
    """

    @plumb
    def __call__(_next, self, model, request):
        ajax_form_fiddle(request, '.principal_form', 'inner')
        return _next(self, model, request)



from cone.app import root
from cone.app.model import Metadata
from cone.app.model import Properties
from cone.ugm import testing
from cone.ugm import ugm_user_acl
from cone.ugm.model.user import User
from node.ext.ugm.interfaces import IUser
import unittest


class TestModelUser(unittest.TestCase):
    layer = testing.ugm_layer

    @testing.principals(
        users={
            'user_1': {},
        })
    def test_user(self):
        # User node
        users = root['users']
        user = users['user_1']
        self.assertTrue(isinstance(user, User))
        self.assertEqual(user.name, 'user_1')

        # Properties
        self.assertTrue(isinstance(user.properties, Properties))

        # Metadata
        md = user.metadata
        self.assertTrue(isinstance(md, Metadata))
        self.assertEqual(md.title, 'user_node')
        self.assertEqual(md.description, 'user_node_description')

        # UGM backend user node is available at ``model``
        self.assertTrue(IUser.providedBy(user.model))

        # Attributes of the user are wrapped
        self.assertTrue(user.attrs is user.model.attrs)

        # ACL
        self.assertEqual(user.__acl__, ugm_user_acl)



from cone.ugm.model.group import Group
from cone.ugm.model.user import User
from pyramid.i18n import get_localizer
from pyramid.i18n import TranslationStringFactory
from pyramid.view import view_config


_ = TranslationStringFactory('cone.ugm')


###############################################################################
# local manager manage membership validation
###############################################################################

LM_TARGET_GID_NOT_ALLOWED = 0
LM_TARGET_UID_NOT_ALLOWED = 1
LM_TARGET_GID_IS_DEFAULT = 2


class ManageMembershipError(Exception):

    def __init__(self, reason, data):
        self.reason = reason
        self.data = data


def validate_add_users_to_groups(model, user_ids, group_ids):
    if not model.local_manager_consider_for_user:
        return
    lm_gids = model.local_manager_target_gids
    for group_id in group_ids:
        if group_id not in lm_gids:
            raise ManageMembershipError(LM_TARGET_GID_NOT_ALLOWED, group_id)
    lm_uids = model.local_manager_target_uids
    for user_id in user_ids:
        if user_id not in lm_uids:
            raise ManageMembershipError(LM_TARGET_UID_NOT_ALLOWED, user_id)


def validate_remove_users_from_groups(model, user_ids, group_ids):
    if not model.local_manager_consider_for_user:
        return
    lm_gids = model.local_manager_target_gids
    for group_id in group_ids:
        if group_id not in lm_gids:
            raise ManageMembershipError(LM_TARGET_GID_NOT_ALLOWED, group_id)
    lm_uids = model.local_manager_target_uids
    for user_id in user_ids:
        if user_id not in lm_uids:
            raise ManageMembershipError(LM_TARGET_UID_NOT_ALLOWED, user_id)
    adm_gid = model.local_manager_gid
    for group_id in group_ids:
        if model.local_manager_is_default(adm_gid, group_id):
            raise ManageMembershipError(LM_TARGET_GID_IS_DEFAULT, group_id)


###############################################################################
# Actions for User application node
###############################################################################

@view_config(
    name='delete_item',
    accept='application/json',
    renderer='json',
    context=User,
    permission='delete_user')
def delete_user_action(model, request):
    """Delete user from database.
    """
    try:
        users = model.parent.backend
        uid = model.model.name
        del users[uid]
        users()
        model.parent.invalidate()
        localizer = get_localizer(request)
        message = localizer.translate(_(
            'delete_user_from_database',
            default="Deleted user '${uid}' from database.",
            mapping={'uid': uid}
        ))
        return {
            'success': True,
            'message': message
        }
    except Exception as e:
        return {
            'success': False,
            'message': str(e)
        }


@view_config(
    name='add_item',
    accept='application/json',
    renderer='json',
    context=User,
    permission='manage_membership')
def user_add_to_group_action(model, request):
    """Add user to group.
    """
    group_id = request.params.get('id')
    if not group_id:
        group_ids = request.params.getall('id[]')
    else:
        group_ids = [group_id]
    try:
        user = model.model
        validate_add_users_to_groups(model, [user.id], group_ids)
        groups = user.root.groups
        for group_id in group_ids:
            groups[group_id].add(user.name)
        groups()
        model.parent.invalidate(user.name)
        localizer = get_localizer(request)
        message = localizer.translate(_(
            'added_user_to_group',
            default="Added user '${uid}' to group '${gid}'.",
            mapping={
                'uid': user.id,
                'gid': ', '.join(group_ids)
            }
        ))
        return {
            'success': True,
            'message': message
        }
    except ManageMembershipError as e:
        if e.reason is not LM_TARGET_GID_NOT_ALLOWED:
            raise Exception(u"Unknown ManageMembershipError reason.")
        localizer = get_localizer(request)
        message = localizer.translate(_(
            'lm_add_target_gid_not_allowed',
            default=(
                "Failed adding user '${uid}' to group '${gid}'. "
                "Manage membership denied for target group."
            ),
            mapping={
                'uid': user.id,
                'gid': e.data
            }
        ))
        return {
            'success': False,
            'message': message
        }
    except Exception as e:
        return {
            'success': False,
            'message': str(e)
        }


@view_config(
    name='remove_item',
    accept='application/json',
    renderer='json',
    context=User,
    permission='manage_membership')
def user_remove_from_group_action(model, request):
    """Remove user from group.
    """
    group_id = request.params.get('id')
    if not group_id:
        group_ids = request.params.getall('id[]')
    else:
        group_ids = [group_id]
    try:
        user = model.model
        validate_remove_users_from_groups(model, [user.id], group_ids)
        groups = user.root.groups
        for group_id in group_ids:
            del groups[group_id][user.name]
        groups()
        model.parent.invalidate(user.name)
        localizer = get_localizer(request)
        message = localizer.translate(_(
            'removed_user_from_group',
            default="Removed user '${uid}' from group '${gid}'.",
            mapping={
                'uid': user.id,
                'gid': ', '.join(group_ids)
            }
        ))
        return {
            'success': True,
            'message': message
        }
    except ManageMembershipError as e:
        localizer = get_localizer(request)
        if e.reason is LM_TARGET_GID_NOT_ALLOWED:
            message = localizer.translate(_(
                'lm_remove_target_gid_not_allowed',
                default=(
                    "Failed removing user '${uid}' from group '${gid}'. "
                    "Manage membership denied for target group."),
                mapping={
                    'uid': user.id,
                    'gid': e.data
                }
            ))
        elif e.reason is LM_TARGET_GID_IS_DEFAULT:
            message = localizer.translate(_(
                'lm_remove_target_gid_is_default',
                default=(
                    "Failed removing user '${uid}' from group '${gid}'. "
                    "Target group is default group of user."
                ),
                mapping={
                    'uid': user.id,
                    'gid': e.data
                }
            ))
        else:
            raise Exception(u"Unknown ManageMembershipError reason.")
        return {
            'success': False,
            'message': message
        }
    except Exception as e:
        return {
            'success': False,
            'message': str(e)
        }


###############################################################################
# Actions for Group application node
###############################################################################

@view_config(
    name='delete_item',
    accept='application/json',
    renderer='json',
    context=Group,
    permission='delete_group')
def delete_group_action(model, request):
    """Delete group from database.
    """
    try:
        groups = model.parent.backend
        uid = model.model.name
        del groups[uid]
        groups()
        model.parent.invalidate()
    except Exception as e:
        return {
            'success': False,
            'message': str(e)
        }
    localizer = get_localizer(request)
    message = localizer.translate(_(
        'deleted_group',
        default='Deleted group from database'
    ))
    return {
        'success': True,
        'message': message
    }


@view_config(
    name='add_item',
    accept='application/json',
    renderer='json',
    context=Group,
    permission='manage_membership')
def group_add_user_action(model, request):
    """Add user to group.
    """
    user_id = request.params.get('id')
    if not user_id:
        user_ids = request.params.getall('id[]')
    else:
        user_ids = [user_id]
    try:
        group = model.model
        validate_add_users_to_groups(model, user_ids, [group.id])
        for user_id in user_ids:
            group.add(user_id)
        group()
        model.parent.invalidate(group.name)
        localizer = get_localizer(request)
        message = localizer.translate(_(
            'added_user_to_group',
            default="Added user '${uid}' to group '${gid}'.",
            mapping={
                'uid': ', '.join(user_ids),
                'gid': group.id
            }
        ))
        return {
            'success': True,
            'message': message
        }
    except ManageMembershipError as e:
        if e.reason is not LM_TARGET_UID_NOT_ALLOWED:
            raise Exception(u"Unknown ManageMembershipError reason.")
        localizer = get_localizer(request)
        message = localizer.translate(_(
            'lm_add_target_uid_not_allowed',
            default=(
                "Failed adding user '${uid}' to group '${gid}'. "
                "Manage membership denied for user."
            ),
            mapping={
                'uid': e.data,
                'gid': group.id
            }
        ))
        return {
            'success': False,
            'message': message
        }
    except Exception as e:
        return {
            'success': False,
            'message': str(e)
        }


@view_config(
    name='remove_item',
    accept='application/json',
    renderer='json',
    context=Group,
    permission='manage_membership')
def group_remove_user_action(model, request):
    """Remove user from group.
    """
    user_id = request.params.get('id')
    if not user_id:
        user_ids = request.params.getall('id[]')
    else:
        user_ids = [user_id]
    try:
        group = model.model
        validate_remove_users_from_groups(model, user_ids, [group.id])
        for user_id in user_ids:
            del group[user_id]
        group()
        model.parent.invalidate(group.name)
        localizer = get_localizer(request)
        message = localizer.translate(_(
            'removed_user_from_group',
            default="Removed user '${uid}' from group '${gid}'.",
            mapping={
                'uid': ', '.join(user_ids),
                'gid': group.id
            }
        ))
        return {
            'success': True,
            'message': message
        }
    except ManageMembershipError as e:
        localizer = get_localizer(request)
        if e.reason is LM_TARGET_UID_NOT_ALLOWED:
            message = localizer.translate(_(
                'lm_remove_target_uid_not_allowed',
                default=(
                    "Failed removing user '${uid}' from group '${gid}'. "
                    "Manage membership denied for user."
                ),
                mapping={
                    'uid': e.data,
                    'gid': group.id
                }
            ))
        elif e.reason is LM_TARGET_GID_IS_DEFAULT:
            message = localizer.translate(_(
                'lm_remove_target_gid_is_default',
                default=(
                    "Failed removing user '${uid}' from group '${gid}'. "
                    "Target group is default group of user."
                ),
                mapping={
                    'uid': ', '.join(user_ids),
                    'gid': e.data
                }
            ))
        else:
            raise Exception(u"Unknown ManageMembershipError reason.")
        return {
            'success': False,
            'message': message
        }
    except Exception as e:
        return {
            'success': False,
            'message': str(e)
        }



from cone.app.browser.utils import make_url
from cone.app.ugm import ugm_backend
from cone.ugm.utils import general_settings
from pyramid.i18n import get_localizer
from pyramid.i18n import TranslationStringFactory
from yafowil.base import ExtractionError
from yafowil.base import factory
from yafowil.base import UNSET
from yafowil.common import generic_extractor
from yafowil.common import ascii_extractor
import itertools


_ = TranslationStringFactory('cone.ugm')


###############################################################################
# Form helper functions
###############################################################################

def default_required_message(request, label):
    localizer = get_localizer(request)
    return _(
        'no_field_value_defined',
        default='No ${field} defined',
        mapping={'field': localizer.translate(_(label, default=label))}
    )


###############################################################################
# Form field factory basics
###############################################################################

def default_form_field_factory(form, label, value, required=False):
    """Default form field factory.

    A form field factory is a callable which gets passed the form tile instance,
    a label and the preset value of a form field, and returns a widget created
    by the yafowil factory.

    The default factory creates a simple text field with no validation
    or whatsoever.

    :param form: The form tile instance.
    :param label: The form field label.
    :param value: The field preset value.
    :return: ``yafowil.base.Widget`` instance created by
        ``yafowil.base.factory``.
    """
    if required and isinstance(required, bool):
        required = default_required_message(form.request, label)
    return factory(
        'field:label:error:text',
        value=value,
        props={
            'label': label,
            'required': required
        })


###############################################################################
# Form field factory registries
###############################################################################

SCOPE_USER = 'user'
SCOPE_GROUP = 'group'
BACKEND_ALL = '__all_backends__'


class _form_field(object):
    """Abstract form field factory registry and decorator.
    """

    scope = None
    """Registration scope."""

    registry = {
        SCOPE_USER: {},
        SCOPE_GROUP: {}
    }
    """Form field factory and attribute name registry by scope and UGM backend
    name.
    """

    def __init__(self, field, backend=BACKEND_ALL):
        """Initialize decorator by attribute name and backend name.

        :param field: The form field name.
        :param backend: UGM backend name the form field factory is valid for.
        """
        assert self.scope
        self.field = field
        self.backend = backend

    def __call__(self, factory):
        """Decorator call. Register the form field factory.

        :param factory: The form field factory callable.
        :return: The form field factory callable.
        """
        scope_reg = self.registry[self.scope]
        backend_reg = scope_reg.setdefault(self.backend, {})
        backend_reg[self.field] = factory
        return factory

    @classmethod
    def factory(cls, field, backend=BACKEND_ALL):
        """Lookup form field factory by field name and backend name.

        :param field: The form field name.
        :param backend: UGM backend name the form field factory is valid for.
        :return: ``FormFieldFactoryProxy`` wrapping the principal node attribute
            name and the form field factory callable.
        """
        assert cls.scope
        scope_reg = cls.registry[cls.scope]
        for backend_name in (backend, BACKEND_ALL):
            backend_reg = scope_reg.setdefault(backend_name, {})
            factory = backend_reg.get(field)
            if factory:
                return factory
        return default_form_field_factory


class user_field(_form_field):
    """Form field factory registry and decorator for user principal form.
    """
    scope = SCOPE_USER


class group_field(_form_field):
    """Form field factory registry and decorator for group principal form.
    """
    scope = SCOPE_GROUP


###############################################################################
# Principal ID form field factories
###############################################################################

class PrincipalExistsExtractor(object):
    """Abstract application model aware yafowil extractor checking whether
    principal ID already exists.
    """

    def __init__(self, model):
        self.model = model

    def __call__(self, widget, data):
        """Check whether principal with ID already exists and raise
        extraction error if so.
        """
        principal_id = data.extracted
        if principal_id is UNSET:
            return principal_id
        try:
            self.model.parent.backend[principal_id]
            raise ExtractionError(self.error_message(principal_id))
        except KeyError:
            return data.extracted

    def error_message(self, principal_id):
        raise NotImplementedError(
            'Abstract ``PrincipalExistsExtractor```does '
            'not implement ``error_message``'
        )


class UserExistsExtractor(PrincipalExistsExtractor):
    """Yafowil extractor checking whether user ID already exists.
    """

    def error_message(self, principal_id):
        return _(
            'user_already_exists',
            default="User ${principal_id} already exists.",
            mapping={'principal_id': principal_id}
        )


class GroupExistsExtractor(PrincipalExistsExtractor):
    """Yafowil extractor checking whether group ID already exists.
    """

    def error_message(self, principal_id):
        return _(
            'group_already_exists',
            default="Group ${principal_id} already exists.",
            mapping={'principal_id': principal_id}
        )


class PrincipalIdFieldFactory(object):
    """Principal ID field factory.

    Creates a form widget which validates an input only contains ASCII
    characters and principal not exists. If edit form, field is not editable.
    """

    def __init__(self, principal_exists_extractor):
        self.principal_exists_extractor = principal_exists_extractor

    def __call__(self, form, label, value):
        return factory(
            'field:*ascii:*exists:label:error:text',
            value=value,
            props={
                'label': label,
                'required': default_required_message(form.request, label),
                'ascii': True
            },
            custom={
                'ascii': {
                    'extractors': [ascii_extractor]
                },
                'exists': {
                    'extractors': [self.principal_exists_extractor(form.model)]
                }
            },
            mode='edit' if form.action_resource == 'add' else 'display'
        )


# register user ID field factory
user_id_field_factory = user_field('id')(
    PrincipalIdFieldFactory(UserExistsExtractor)
)

# register group ID field factory
group_id_field_factory = group_field('id')(
    PrincipalIdFieldFactory(GroupExistsExtractor)
)


###############################################################################
# Login name form field factory
###############################################################################

class LoginNameExtractor(object):
    """Application model aware yafowil extractor checking whether optional
    login name is valid.
    """

    def __init__(self, model, login_attr):
        self.model = model
        self.login_attr = login_attr

    def __call__(self, widget, data):
        """Check whether user login name already exists and raise
        extraction error if so.
        """
        login = generic_extractor(widget, data)
        if not login:
            return login
        res = self.model.parent.backend.search(criteria={
            self.login_attr: login
        })
        # no entries found with same login attribute set.
        if not res:
            return login
        # unchanged login attribute of current user
        if len(res) == 1 and res[0] == self.model.name:
            return login
        message = _(
            'user_login_not_unique',
            default='User login ${login} not unique.',
            mapping={'login': data.extracted}
        )
        raise ExtractionError(message)


@user_field('login')
def login_name_field_factory(form, label, value):
    settings = general_settings(form.model).attrs
    login_attr = settings.users_login_name_attr
    if login_attr == 'login':
        factory = default_form_field_factory
    else:
        factory = user_field.factory(login_attr, backend=ugm_backend.name)
    widget = factory(form, label, value)
    login_extractor = LoginNameExtractor(form.model, login_attr)
    widget.blueprints.append('*login')
    widget.custom['login'] = dict(extractors=[login_extractor])
    widget.extractors.insert(0, ('login', login_extractor))
    widget.mode = 'edit' if login_attr else 'skip'
    return widget


###############################################################################
# Password form field factory
###############################################################################

@user_field('password')
def password_field_factory(form, label, value):
    return factory(
        'field:label:error:password',
        value=value,
        props={
            'label': label,
            'required': default_required_message(form.request, label),
            'minlength': 6,
            'ascii': True
        })


###############################################################################
# Email form field factory
###############################################################################

@user_field('mail')
@user_field('email')
def email_field_factory(form, label, value):
    return factory(
        'field:label:error:email',
        value=value,
        props={
            'label': label
        })


###############################################################################
# Principal form
###############################################################################

class PrincipalForm(object):
    form_name = None

    @property
    def reserved_attrs(self):
        raise NotImplementedError(
            'Abstract principal form does not implement ``reserved_attrs``'
        )

    @property
    def form_attrmap(self):
        raise NotImplementedError(
            'Abstract principal form does not implement ``form_attrmap``'
        )

    @property
    def field_factory_registry(self):
        raise NotImplementedError(
            'Abstract principal form does not '
            'implement ``field_factory_registry``'
        )

    def prepare(self):
        model = self.model
        request = self.request
        scope = self.action_resource
        self.form = form = factory(
            u'form',
            name=self.form_name,
            props={
                'action': make_url(request, node=model, resource=scope),
            })
        registry = self.field_factory_registry
        backend_name = ugm_backend.name
        form_attrs = itertools.chain(
            self.reserved_attrs.items(),
            self.form_attrmap.items() if self.form_attrmap else []
        )
        for attr_name, label in form_attrs:
            field_factory = registry.factory(attr_name, backend=backend_name)
            value = model.attrs.get(attr_name, UNSET)
            form[attr_name] = field_factory(self, label, value)
        form['save'] = factory(
            'submit',
            props={
                'action': 'save',
                'expression': True,
                'handler': self.save,
                'next': self.next,
                'label': _('save', default='Save')
            })
        if scope == 'add':
            form['cancel'] = factory(
                'submit',
                props={
                    'action': 'cancel',
                    'expression': True,
                    'handler': None,
                    'next': self.next,
                    'label': _('cancel', default='Cancel'),
                    'skip': True
                })



from cone.app import get_root
from cone.app.model import XMLProperties
from cone.ugm import testing
from cone.ugm.settings import GeneralSettings
from cone.ugm.settings import UGMSettings
from node.tests import NodeTestCase
import os


class TestModelSettings(NodeTestCase):
    layer = testing.ugm_layer

    @testing.temp_directory
    def test_UGMSettings(self, tempdir):
        path = os.path.join(tempdir, 'settings.xml')

        class MyUGMSettings(UGMSettings):
            config_file = path
            initialize_ugm_on_invalidate = False

        settings = MyUGMSettings()
        expected = 'Configuration file {} not exists.'.format(path)
        err = self.expect_error(ValueError, lambda: settings.attrs)
        self.assertEqual(str(err), expected)

        with open(path, 'w') as f:
            f.write('<properties />')

        attrs = settings.attrs
        self.assertTrue(isinstance(attrs, XMLProperties))

        attrs.foo = 'foo'
        settings()

        with open(path, 'r') as f:
            content = f.read()
        expected = '<properties>\n  <foo>foo</foo>\n</properties>\n'
        self.assertEqual(content, expected)

        self.assertTrue(attrs is settings.attrs)
        settings.invalidate()
        self.assertFalse(attrs is settings.attrs)

    @testing.invalidate_settings
    def test_UGMGeneralSettings(self):
        settings = get_root()['settings']['ugm_general']

        self.assertTrue(isinstance(settings, GeneralSettings))

        md = settings.metadata
        self.assertEqual(md.title, 'ugm_settings_node')
        self.assertEqual(md.description, 'ugm_settings_node_description')

        attrs = settings.attrs
        self.assertEqual(sorted(attrs.keys()), [
            'groups_form_attrmap',
            'groups_listing_columns',
            'groups_listing_default_column',
            'roles_principal_roles_enabled',
            'user_id_autoincrement',
            'user_id_autoincrement_prefix',
            'user_id_autoincrement_start',
            'users_account_expiration',
            'users_expires_attr',
            'users_expires_unit',
            'users_exposed_attributes',
            'users_form_attrmap',
            'users_listing_columns',
            'users_listing_default_column',
            'users_local_management_enabled',
            'users_login_name_attr',
            'users_portrait',
            'users_portrait_accept',
            'users_portrait_attr',
            'users_portrait_height',
            'users_portrait_width',
        ])

        self.assertTrue(attrs is settings.attrs)
        settings.invalidate()
        self.assertFalse(attrs is settings.attrs)



from cone.app.browser.utils import make_query
from cone.app.browser.utils import make_url
from cone.tile import Tile
from cone.tile import tile
from cone.ugm.browser.columns import Column
from cone.ugm.browser.listing import PrincipalsListing
from cone.ugm.model.users import Users
from pyramid.i18n import TranslationStringFactory
import logging


logger = logging.getLogger('cone.ugm')
_ = TranslationStringFactory('cone.ugm')


@tile(
    name='leftcolumn',
    path='templates/principals_left_column.pt',
    interface=Users,
    permission='view')
class UsersLeftColumn(Tile):
    add_label = _('add_user', default='Add User')

    @property
    def add_target(self):
        return make_url(
            self.request,
            node=self.model.root['users'],
            query=make_query(factory=u'user')
        )

    @property
    def can_add(self):
        return self.request.has_permission('add_user', self.model)


@tile(
    name='rightcolumn',
    path='templates/principals_right_column.pt',
    interface=Users,
    permission='view')
class UsersRightColumn(Column):

    @property
    def principal_id(self):
        return self.request.params.get('pid')

    @property
    def principal_form(self):
        return self._render(self.model[self.principal_id], 'editform')

    @property
    def principal_target(self):
        return make_url(self.request, node=self.model[self.principal_id])


@tile(
    name='columnlisting',
    path='templates/column_listing.pt',
    interface=Users,
    permission='view')
class UsersColumnListing(PrincipalsListing):
    slot = 'leftlisting'
    list_columns = PrincipalsListing.user_list_columns
    listing_attrs = PrincipalsListing.user_attrs
    localmanager_ids = PrincipalsListing.user_localmanager_ids
    sort_attr = PrincipalsListing.user_default_sort_column
    css = 'users'
    batchname = 'leftbatch'
    delete_label = _('delete_user', default='Delete User')
    delete_permission = 'delete_user'

    @property
    def current_id(self):
        return self.request.params.get('pid')



from cone.app import layout_config
from cone.ugm import UGMLayoutConfig
from cone.ugm.model.group import Group
from cone.ugm.model.groups import Groups
from cone.ugm.model.user import User
from cone.ugm.model.users import Users
from pyramid.testing import DummyRequest
import unittest


class TestLayout(unittest.TestCase):

    def test_layout(self):
        config = layout_config.lookup(
            model=Group(None, None, None),
            request=DummyRequest()
        )
        self.assertIsInstance(config, UGMLayoutConfig)

        config = layout_config.lookup(model=Groups(), request=DummyRequest())
        self.assertIsInstance(config, UGMLayoutConfig)

        config = layout_config.lookup(
            model=User(None, None, None),
            request=DummyRequest()
        )
        self.assertIsInstance(config, UGMLayoutConfig)

        config = layout_config.lookup(model=Users(), request=DummyRequest())
        self.assertIsInstance(config, UGMLayoutConfig)

        self.assertTrue(config.mainmenu)
        self.assertFalse(config.mainmenu_fluid)
        self.assertFalse(config.livesearch)
        self.assertTrue(config.personaltools)
        self.assertFalse(config.columns_fluid)
        self.assertFalse(config.pathbar)
        self.assertEqual(config.sidebar_left, [])
        self.assertEqual(config.sidebar_left_grid_width, 0)
        self.assertEqual(config.content_grid_width, 12)



from cone.app import get_root
from cone.app import root
from cone.tile import render_tile
from cone.tile.tests import TileTestCase
from cone.ugm import testing
from cone.ugm.browser.settings import GeneralSettingsForm
from cone.ugm.settings import ugm_cfg
from pyramid.httpexceptions import HTTPForbidden
import os


class TestBrowserSettings(TileTestCase):
    layer = testing.ugm_layer

    @testing.custom_config_path
    @testing.temp_directory
    def test_GeneralSettingsForm(self, tempdir):
        config_file = os.path.join(tempdir, 'ugm.xml')
        ugm_cfg.ugm_settings = config_file
        with open(config_file, 'w') as f:
            f.write('<properties></properties>')

        model = get_root()['settings']['ugm_general']
        request = self.layer.new_request()

        tile = GeneralSettingsForm()
        tile.model = model
        tile.request = request
        tile.prepare()

        form = tile.form
        self.assertEqual(form.keys(), [
            'users_heading',
            'users_account_expiration',
            'users_expires_attr',
            'users_expires_unit',
            'user_id_autoincrement',
            'user_id_autoincrement_prefix',
            'user_id_autoincrement_start',
            'users_portrait',
            'users_portrait_attr',
            'users_portrait_accept',
            'users_portrait_width',
            'users_portrait_height',
            'users_local_management_enabled',
            'users_login_name_attr',
            'users_exposed_attributes',
            'users_form_attrmap',
            'users_listing_columns',
            'users_listing_default_column',
            'groups_heading',
            'groups_form_attrmap',
            'groups_listing_columns',
            'groups_listing_default_column',
            'roles_heading',
            'roles_principal_roles_enabled',
            'save'
        ])

    @testing.principals(
        users={
            'editor': {},
            'manager': {},
        },
        roles={
            'editor': ['editor'],
            'manager': ['manager']
        })
    def test_general_settings_tiles(self):
        general_settings = root['settings']['ugm_general']
        request = self.layer.new_request()

        # Unauthenticated content tile raises error
        self.expectError(
            HTTPForbidden,
            render_tile,
            general_settings,
            request,
            'content'
        )

        # Form tile raise if not manager
        with self.layer.authenticated('editor'):
            self.expectError(
                HTTPForbidden,
                render_tile,
                general_settings,
                request,
                'editform'
            )

        # Authenticate and render tile
        with self.layer.authenticated('manager'):
            res = render_tile(general_settings, request, 'editform')
        expected = 'form action="http://example.com/settings/ugm_general/edit"'
        self.assertTrue(res.find(expected) > -1)

    @testing.principals(
        users={
            'editor': {},
            'manager': {},
        },
        roles={
            'editor': ['editor'],
            'manager': ['manager']
        })
    def test_localmanager_settings_tiles(self):
        lm_settings = root['settings']['ugm_localmanager']
        request = self.layer.new_request()

        # Unauthenticated content tile raises error
        self.expectError(
            HTTPForbidden,
            render_tile,
            lm_settings,
            request,
            'content'
        )

        # Form tile raise if not manager
        with self.layer.authenticated('editor'):
            self.expectError(
                HTTPForbidden,
                render_tile,
                lm_settings,
                request,
                'editform'
            )

        # Authenticate and render tile
        with self.layer.authenticated('manager'):
            res = render_tile(lm_settings, request, 'editform')
        expected = 'form action="http://example.com/settings/ugm_localmanager/edit"'
        self.assertTrue(res.find(expected) > -1)



from cone.app import root
from cone.app.model import Metadata
from cone.app.model import Properties
from cone.ugm import testing
from cone.ugm import ugm_default_acl
from cone.ugm.model.group import Group
from node.ext.ugm.interfaces import IGroup
import unittest


class TestModelGroup(unittest.TestCase):
    layer = testing.ugm_layer

    @testing.principals(
        groups={
            'group_1': {},
        })
    def test_group(self):
        # Group node
        groups = root['groups']
        group = groups['group_1']
        self.assertTrue(isinstance(group, Group))
        self.assertEqual(group.name, 'group_1')

        # Properties
        self.assertTrue(isinstance(group.properties, Properties))

        # Metadata
        md = group.metadata
        self.assertTrue(isinstance(md, Metadata))
        self.assertEqual(md.title, 'group_node')
        self.assertEqual(md.description, 'group_node_description')

        # UGM backend group node is available at ``model``
        self.assertTrue(IGroup.providedBy(group.model))

        # Attributes of the group are wrapped
        self.assertTrue(group.attrs is group.model.attrs)

        # ACL
        self.assertEqual(group.__acl__, ugm_default_acl)



from cone.app import get_root
from cone.app.model import BaseNode
from cone.tile import Tile
from cone.tile.tests import TileTestCase
from cone.ugm import testing
from cone.ugm.browser.principal import _form_field
from cone.ugm.browser.principal import default_form_field_factory
from cone.ugm.browser.principal import default_required_message
from cone.ugm.browser.principal import email_field_factory
from cone.ugm.browser.principal import group_field
from cone.ugm.browser.principal import group_id_field_factory
from cone.ugm.browser.principal import GroupExistsExtractor
from cone.ugm.browser.principal import login_name_field_factory
from cone.ugm.browser.principal import LoginNameExtractor
from cone.ugm.browser.principal import password_field_factory
from cone.ugm.browser.principal import PrincipalExistsExtractor
from cone.ugm.browser.principal import PrincipalForm
from cone.ugm.browser.principal import PrincipalIdFieldFactory
from cone.ugm.browser.principal import user_field
from cone.ugm.browser.principal import user_id_field_factory
from cone.ugm.browser.principal import UserExistsExtractor
from cone.ugm.utils import general_settings
from node.utils import UNSET
from odict import odict
from pyramid.i18n import get_localizer
from yafowil.base import ExtractionError
from yafowil.base import factory
from yafowil.common import ascii_extractor


class TestBrowserPrincipal(TileTestCase):
    layer = testing.ugm_layer

    def test_default_required_message(self):
        request = self.layer.new_request()
        # use inexistent locale to ensure message catalog bypass
        request._LOCALE_ = 'foo'
        message = default_required_message(request, 'Name')
        localizer = get_localizer(request)
        self.assertEqual(localizer.translate(message), 'No Name defined')

    def test_default_form_field_factory(self):
        form = Tile()
        form.request = self.layer.new_request()
        label = 'Field Label'
        value = 'Field Value'
        widget = default_form_field_factory(form, label, value)
        self.assertEqual(widget.getter, 'Field Value')
        self.assertEqual(widget.blueprints, ['field', 'label', 'error', 'text'])
        self.assertEqual(widget.properties, {
            'label': 'Field Label',
            'required': False
        })
        widget = default_form_field_factory(form, label, value, required=True)
        self.assertEqual(widget.properties, {
            'label': 'Field Label',
            'required': 'no_field_value_defined'
        })

    def test__form_field(self):
        self.expectError(AssertionError, _form_field, 'field')
        self.expectError(AssertionError, _form_field.factory, 'field')

        SCOPE = 'form'
        _form_field.registry[SCOPE] = {}

        class form_field(_form_field):
            scope = SCOPE

        @form_field('field')
        def form_field_factory(form, label, value):
            pass

        @form_field('field', backend='backend')
        def backend_form_field_factory(form, label, value):
            pass

        self.assertEqual(_form_field.registry[SCOPE], {
            '__all_backends__': {
                'field': form_field_factory,
            },
            'backend': {
                'field': backend_form_field_factory,
            }
        })

        factory = form_field.factory('field')
        self.assertEqual(factory, form_field_factory)

        factory = form_field.factory('other_field')
        self.assertEqual(factory, default_form_field_factory)

        factory = form_field.factory('field', backend='backend')
        self.assertEqual(factory, backend_form_field_factory)

        factory = form_field.factory('other_field', backend='backend')
        self.assertEqual(factory, default_form_field_factory)

        factory = form_field.factory('field', backend='other')
        self.assertEqual(factory, form_field_factory)

        factory = form_field.factory('other_field', backend='other')
        self.assertEqual(factory, default_form_field_factory)

        del _form_field.registry[SCOPE]

    def test_PrincipalExistsExtractor(self):
        # Abstract error_message
        extractor = PrincipalExistsExtractor(BaseNode())
        self.expectError(NotImplementedError, extractor.error_message, 'pid')

        # dummy model
        class Principals(BaseNode):
            backend = {'pid': object()}

        # dummy extractor
        class MyPrincipalExistsExtractor(PrincipalExistsExtractor):
            def error_message(self, principal_id):
                return 'Principal {} already exists'.format(principal_id)

        # principals container, add model and form widget
        principals = Principals(name='principals')
        add_model = BaseNode(parent=principals)
        extractor = MyPrincipalExistsExtractor(add_model)
        widget = factory(
            '*exists:text',
            name='principal_id',
            custom={'exists': {'extractors': [extractor]}}
        )

        # test already exists
        request = {'principal_id': 'pid'}
        data = widget.extract(request=request)
        self.assertTrue(data.has_errors)
        self.assertEqual(
            data.errors,
            [ExtractionError('Principal pid already exists')]
        )

        # test not already exists
        request = {'principal_id': 'new_pid'}
        data = widget.extract(request=request)
        self.assertFalse(data.has_errors)
        self.assertEqual(data.extracted, 'new_pid')

    @testing.principals(
        users={
            'user_exists': {}
        })
    def test_UserExistsExtractor(self):
        root = get_root()
        users = root['users']
        add_model = BaseNode(parent=users)
        extractor = UserExistsExtractor(add_model)
        widget = factory(
            '*exists:text',
            name='user_id',
            custom={'exists': {'extractors': [extractor]}}
        )

        # test already exists
        request = {'user_id': 'user_exists'}
        data = widget.extract(request=request)
        self.assertTrue(data.has_errors)
        self.assertEqual(
            data.errors,
            [ExtractionError('user_already_exists')]
        )

        # test not already exists
        request = {'user_id': 'new_user_id'}
        data = widget.extract(request=request)
        self.assertFalse(data.has_errors)
        self.assertEqual(data.extracted, 'new_user_id')

    @testing.principals(
        groups={
            'group_exists': {}
        })
    def test_GroupExistsExtractor(self):
        root = get_root()
        groups = root['groups']
        add_model = BaseNode(parent=groups)
        extractor = GroupExistsExtractor(add_model)
        widget = factory(
            '*exists:text',
            name='group_id',
            custom={'exists': {'extractors': [extractor]}}
        )

        # test already exists
        request = {'group_id': 'group_exists'}
        data = widget.extract(request=request)
        self.assertTrue(data.has_errors)
        self.assertEqual(
            data.errors,
            [ExtractionError('group_already_exists')]
        )

        # test not already exists
        request = {'group_id': 'new_group_id'}
        data = widget.extract(request=request)
        self.assertFalse(data.has_errors)
        self.assertEqual(data.extracted, 'new_group_id')

    def test_PrincipalIdFieldFactory(self):
        class PrincipalAddForm(Tile):
            action_resource = 'add'

        class PrincipalEditForm(Tile):
            action_resource = 'edit'

        factory = PrincipalIdFieldFactory(PrincipalExistsExtractor)

        # principal add form
        form = PrincipalAddForm()
        form.model = BaseNode()
        form.request = self.layer.new_request()
        widget = factory(form, 'Principal ID', UNSET)
        self.assertEqual(widget.blueprints, [
            'field', '*ascii', '*exists', 'label', 'error', 'text'
        ])
        self.assertEqual(widget.getter, UNSET)
        self.assertEqual(widget.properties, {
            'label': 'Principal ID',
            'required': 'no_field_value_defined',
            'ascii': True
        })
        self.assertEqual(
            widget.custom['ascii']['extractors'],
            [ascii_extractor]
        )
        self.assertTrue(isinstance(
            widget.custom['exists']['extractors'][0],
            PrincipalExistsExtractor
        ))
        self.assertEqual(widget.mode, 'edit')

        # principal edit form
        form = PrincipalEditForm()
        form.model = BaseNode()
        form.request = self.layer.new_request()
        widget = factory(form, 'Principal ID', 'pid')
        self.assertEqual(widget.getter, 'pid')
        self.assertEqual(widget.properties, {
            'label': 'Principal ID',
            'required': 'no_field_value_defined',
            'ascii': True
        })
        self.assertEqual(
            widget.custom['ascii']['extractors'],
            [ascii_extractor]
        )
        self.assertTrue(isinstance(
            widget.custom['exists']['extractors'][0],
            PrincipalExistsExtractor
        ))
        self.assertEqual(widget.mode, 'display')

    def test_user_id_field_factory(self):
        factory = user_field.factory('id')
        self.assertTrue(isinstance(factory, PrincipalIdFieldFactory))
        self.assertEqual(factory, user_id_field_factory)
        self.assertEqual(
            factory.principal_exists_extractor,
            UserExistsExtractor
        )

    def test_group_id_field_factory(self):
        factory = group_field.factory('id')
        self.assertTrue(isinstance(factory, PrincipalIdFieldFactory))
        self.assertEqual(factory, group_id_field_factory)
        self.assertEqual(
            factory.principal_exists_extractor,
            GroupExistsExtractor
        )

    def test_LoginNameExtractor(self):
        # dummy backend
        class UsersBackend(object):
            def search(self, criteria):
                if criteria['login_attr'] == 'Login Name':
                    return ['user_name']

        # dummy model
        class Users(BaseNode):
            backend = UsersBackend()

        # adding. users container, add model and form widget
        users = Users(name='users')
        add_model = BaseNode(parent=users)
        extractor = LoginNameExtractor(add_model, 'login_attr')
        widget = factory(
            '*login:text',
            name='login_name',
            custom={'login': {'extractors': [extractor]}}
        )

        # test already exists
        request = {'login_name': 'Login Name'}
        data = widget.extract(request=request)
        self.assertTrue(data.has_errors)
        self.assertEqual(
            data.errors,
            [ExtractionError('user_login_not_unique')]
        )

        # test not already exists
        request = {'login_name': 'Other Login Name'}
        data = widget.extract(request=request)
        self.assertFalse(data.has_errors)
        self.assertEqual(data.extracted, 'Other Login Name')

        # test empty login name
        request = {'login_name': ''}
        data = widget.extract(request=request)
        self.assertFalse(data.has_errors)
        self.assertEqual(data.extracted, '')

        # editing. users container, edit model and form widget
        edit_model = BaseNode(name='user_name', parent=users)
        extractor = LoginNameExtractor(edit_model, 'login_attr')
        widget = factory(
            '*login:text',
            name='login_name',
            custom={'login': {'extractors': [extractor]}}
        )

        # test login name belongs to edited node
        request = {'login_name': 'Login Name'}
        data = widget.extract(request=request)
        self.assertFalse(data.has_errors)
        self.assertEqual(data.extracted, 'Login Name')

        request = {'login_name': 'Other Login Name'}
        data = widget.extract(request=request)
        self.assertFalse(data.has_errors)
        self.assertEqual(data.extracted, 'Other Login Name')

        # test login name belongs to another user
        edit_model.__name__ = 'other_user_name'
        request = {'login_name': 'Login Name'}
        data = widget.extract(request=request)
        self.assertTrue(data.has_errors)
        self.assertEqual(
            data.errors,
            [ExtractionError('user_login_not_unique')]
        )

    @testing.invalidate_settings
    def test_login_name_field_factory(self):
        factory = user_field.factory('login')
        self.assertEqual(factory, login_name_field_factory)

        users = get_root()['users']
        user = BaseNode(name='user', parent=users)
        form = Tile()
        form.model = user
        form.request = self.layer.new_request()

        settings = general_settings(users)
        settings.attrs.users_login_name_attr = ''

        widget = factory(form, 'Login Name', UNSET)
        self.assertEqual(widget.getter, UNSET)
        self.assertEqual(widget.blueprints, [
            'field', 'label', 'error', 'text', '*login'
        ])
        self.assertEqual(widget.properties, {
            'label': 'Login Name',
            'required': False
        })
        self.assertTrue(isinstance(
            widget.custom['login']['extractors'][0],
            LoginNameExtractor
        ))
        self.assertEqual(widget.mode, 'skip')

        settings.attrs.users_login_name_attr = 'login'
        widget = factory(form, 'Login Name', UNSET)
        self.assertEqual(widget.mode, 'edit')

        settings.attrs.users_login_name_attr = 'mail'
        widget = factory(form, 'Login Name', UNSET)
        self.assertEqual(widget.blueprints, [
            'field', 'label', 'error', 'email', '*login'
        ])

    def test_password_field_factory(self):
        factory = user_field.factory('password')
        self.assertEqual(factory, password_field_factory)

        form = Tile()
        form.request = self.layer.new_request()
        widget = factory(form, 'User Password', UNSET)
        self.assertEqual(widget.blueprints, [
            'field', 'label', 'error', 'password'
        ])
        self.assertEqual(widget.getter, UNSET)
        self.assertEqual(widget.properties, {
            'ascii': True,
            'label': 'User Password',
            'minlength': 6,
            'required': 'no_field_value_defined'
        })

    def test_email_field_factory(self):
        factory = user_field.factory('email')
        self.assertEqual(factory, email_field_factory)

        form = Tile()
        form.request = self.layer.new_request()
        widget = factory(form, 'Email address', UNSET)
        self.assertEqual(widget.blueprints, [
            'field', 'label', 'error', 'email'
        ])
        self.assertEqual(widget.getter, UNSET)
        self.assertEqual(widget.properties, {'label': 'Email address'})

    def test_principal_form(self):
        SCOPE = 'principal'
        _form_field.registry[SCOPE] = {}

        class form_field(_form_field):
            scope = SCOPE

        class DummyPrincipalForm(PrincipalForm):
            form_name = 'principal'
            reserved_attrs = odict([('reserved_field', 'reserver_field')])
            form_attrmap = odict([('form_field', 'Form Field')])
            field_factory_registry = form_field

            def save(self, widget, data):
                pass

            def next(self, request):
                pass

        class PrincipalAddForm(DummyPrincipalForm):
            action_resource = 'add'

        class PrincipalEditForm(DummyPrincipalForm):
            action_resource = 'edit'

        # users = get_root()['users']
        model = BaseNode(name='user', parent=None)
        request = self.layer.new_request()

        form = PrincipalAddForm()
        form.model = model
        form.request = request
        form.prepare()

        self.checkOutput("""
        <class 'yafowil.base.Widget'>: principal
          <class 'yafowil.base.Widget'>: reserved_field
          <class 'yafowil.base.Widget'>: form_field
          <class 'yafowil.base.Widget'>: save
          <class 'yafowil.base.Widget'>: cancel
        """, form.form.treerepr())

        self.assertEqual(form.form['reserved_field'].getter, UNSET)
        self.assertEqual(form.form['form_field'].getter, UNSET)

        model.attrs['reserved_field'] = 'Reserved Value'
        model.attrs['form_field'] = 'Field Value'

        form = PrincipalEditForm()
        form.model = model
        form.request = request
        form.prepare()

        self.checkOutput("""
        <class 'yafowil.base.Widget'>: principal
          <class 'yafowil.base.Widget'>: reserved_field
          <class 'yafowil.base.Widget'>: form_field
          <class 'yafowil.base.Widget'>: save
        """, form.form.treerepr())

        self.assertEqual(form.form['reserved_field'].getter, 'Reserved Value')
        self.assertEqual(form.form['form_field'].getter, 'Field Value')

        del _form_field.registry[SCOPE]



from cone.app import cfg
from cone.app import layout_config
from cone.app import main_hook
from cone.app import register_config
from cone.app import register_entry
from cone.app.model import LayoutConfig
from cone.app.security import acl_registry
from cone.ugm import browser
from cone.ugm.model.group import Group
from cone.ugm.model.groups import Groups
from cone.ugm.model.user import User
from cone.ugm.model.users import Users
from cone.ugm.settings import GeneralSettings
from cone.ugm.settings import LocalManagerSettings
from cone.ugm.settings import ugm_cfg
from pyramid.security import ALL_PERMISSIONS
from pyramid.security import Allow
from pyramid.security import Deny
from pyramid.security import Everyone
import logging


logger = logging.getLogger('cone.ugm')

# security
management_permissions = [
    'add', 'edit', 'delete',
]
user_management_permissions = [
    'add_user', 'edit_user', 'delete_user', 'manage_expiration',
]
group_management_permissions = [
    'add_group', 'edit_group', 'delete_group',
]
admin_permissions = [
    'view', 'manage_membership', 'view_portrait',
] + management_permissions \
  + user_management_permissions \
  + group_management_permissions
ugm_default_acl = [
    (Allow, 'role:editor', ['view', 'manage_membership']),
    (Allow, 'role:admin', admin_permissions),
    (Allow, 'role:manager', admin_permissions + ['manage']),
    (Allow, Everyone, ['login']),
    (Deny, Everyone, ALL_PERMISSIONS),
]
ugm_user_acl = [
    (Allow, 'system.Authenticated', ['view_portrait']),
] + ugm_default_acl


@layout_config(Group, Groups, User, Users)
class UGMLayoutConfig(LayoutConfig):

    def __init__(self, model=None, request=None):
        super(UGMLayoutConfig, self).__init__(model=model, request=request)
        self.mainmenu = True
        self.mainmenu_fluid = False
        self.livesearch = False
        self.personaltools = True
        self.columns_fluid = False
        self.pathbar = False
        self.sidebar_left = []
        self.sidebar_left_grid_width = 0
        self.content_grid_width = 12


# application startup hooks
@main_hook
def initialize_ugm(config, global_config, settings):
    """Initialize UGM.
    """
    # custom UGM styles
    cfg.merged.css.protected.append((browser.static_resources, 'styles.css'))

    # custom UGM javascript
    cfg.merged.js.protected.append((browser.static_resources, 'ugm.js'))

    # config file locations
    ugm_cfg.ugm_settings = settings.get('ugm.config', '')
    ugm_cfg.lm_settings = settings.get('ugm.localmanager_config', '')

    # UGM settings
    register_config('ugm_general', GeneralSettings)
    register_config('ugm_localmanager', LocalManagerSettings)

    # Users container
    register_entry('users', Users)

    # Groups container
    register_entry('groups', Groups)

    # register default acl's
    # XXX: define permissions referring users, user, groups respective group only
    acl_registry.register(ugm_user_acl, User, 'user')
    acl_registry.register(ugm_default_acl, Users, 'users')
    acl_registry.register(ugm_default_acl, Group, 'group')
    acl_registry.register(ugm_default_acl, Groups, 'groups')

    # add translation
    config.add_translation_dirs('cone.ugm:locale/')

    # static resources
    config.add_view(browser.static_resources, name='cone.ugm.static')

    # scan browser package
    config.scan(browser)



from cone.app import compat
from cone.app.browser.ajax import AjaxAction
from cone.app.browser.authoring import ContentAddForm
from cone.app.browser.authoring import ContentEditForm
from cone.app.browser.form import Form
from cone.app.browser.utils import make_query
from cone.app.browser.utils import make_url
from cone.app.ugm import ugm_backend
from cone.tile import Tile
from cone.tile import tile
from cone.ugm.browser.authoring import AddFormFiddle
from cone.ugm.browser.authoring import EditFormFiddle
from cone.ugm.browser.autoincrement import AutoIncrementForm
from cone.ugm.browser.expires import ExpirationForm
from cone.ugm.browser.listing import ColumnListing
from cone.ugm.browser.portrait import PortraitForm
from cone.ugm.browser.principal import PrincipalForm
from cone.ugm.browser.principal import user_field
from cone.ugm.browser.roles import PrincipalRolesForm
from cone.ugm.model.user import User
from cone.ugm.utils import general_settings
from odict import odict
from plumber import plumbing
from pyramid.i18n import TranslationStringFactory
from webob.exc import HTTPFound
from yafowil.base import UNSET
import fnmatch
import itertools


_ = TranslationStringFactory('cone.ugm')


@tile(
    name='leftcolumn',
    path='templates/principal_left_column.pt',
    interface=User,
    permission='view')
class UserLeftColumn(Tile):

    @property
    def principals_target(self):
        query = make_query(pid=self.model.name)
        return make_url(self.request, node=self.model.parent, query=query)


@tile(
    name='rightcolumn',
    path='templates/principal_right_column.pt',
    interface=User,
    permission='view')
class UserRightColumn(Tile):
    pass


class GroupsListing(ColumnListing):
    related_only = False

    @property
    def listing_items(self):
        appuser = self.model
        user = appuser.model
        groups = user.groups
        related_ids = [g.name for g in groups]
        # Always True if we list members only, otherwise will be set
        # in the loop below
        related = self.related_only
        if not related:
            groups = self.model.root['groups'].backend.values()
        # reduce for local manager
        if self.model.local_manager_consider_for_user:
            local_gids = self.model.local_manager_target_gids
            groups = [g for g in groups if g.name in local_gids]
        attrlist = self.group_attrs
        sort_attr = self.group_default_sort_column
        filter_term = self.unquoted_param_value('filter')
        can_change = self.request.has_permission(
            'manage_membership',
            self.model.parent
        )
        ret = list()
        for group in groups:
            attrs = group.attrs
            # reduce by search term
            if filter_term:
                s_attrs = [attrs[attr] for attr in attrlist]
                if not fnmatch.filter(s_attrs, filter_term):
                    continue
            gid = group.name
            item_target = make_url(self.request, path=group.path[1:])
            action_query = make_query(id=gid)
            action_target = make_url(
                self.request,
                node=appuser,
                query=action_query
            )
            if not self.related_only:
                related = gid in related_ids
            actions = list()
            if can_change:
                action_id = 'add_item'
                action_enabled = not bool(related)
                action_title = _(
                    'add_user_to_selected_group',
                    default='Add user to selected group'
                )
                add_item_action = self.create_action(
                    action_id,
                    action_enabled,
                    action_title,
                    action_target
                )
                actions.append(add_item_action)
                action_id = 'remove_item'
                action_enabled = bool(related)
                action_title = _(
                    'remove_user_from_selected_group',
                    default='Remove user from selected group'
                )
                remove_item_action = self.create_action(
                    action_id,
                    action_enabled,
                    action_title,
                    action_target
                )
                actions.append(remove_item_action)
            vals = [self.extract_raw(attrs, attr) for attr in attrlist]
            sort = self.extract_raw(attrs, sort_attr)
            content = self.item_content(*vals)
            current = False
            item = self.create_item(
                sort,
                item_target,
                content,
                current,
                actions
            )
            ret.append(item)
        return ret


@tile(
    name='columnlisting',
    path='templates/column_listing.pt',
    interface=User,
    permission='view')
class GroupsOfUserColumnListing(GroupsListing):
    slot = 'rightlisting'
    list_columns = ColumnListing.group_list_columns
    css = 'groups'
    related_only = True
    batchname = 'rightbatch'
    display_limit = True
    display_limit_checked = False


@tile(
    name='allcolumnlisting',
    path='templates/column_listing.pt',
    interface=User,
    permission='view')
class AllGroupsColumnListing(GroupsListing):
    slot = 'rightlisting'
    list_columns = ColumnListing.group_list_columns
    css = 'groups'
    batchname = 'rightbatch'
    display_limit = True
    display_limit_checked = True

    @property
    def ajax_action(self):
        return 'allcolumnlisting'


class UserForm(PrincipalForm):
    form_name = 'userform'
    field_factory_registry = user_field

    @property
    def reserved_attrs(self):
        return odict([
            ('id', _('user_id', default='User ID')),
            ('login', _('login_name', default='Login name')),
            ('password', _('password', default='Password'))
        ])

    @property
    def form_attrmap(self):
        return general_settings(self.model).attrs.users_form_attrmap


@tile(name='addform', interface=User, permission='add_user')
@plumbing(
    ContentAddForm,
    PrincipalRolesForm,
    PortraitForm,
    ExpirationForm,
    AutoIncrementForm,
    AddFormFiddle)
class UserAddForm(UserForm, Form):
    show_heading = False
    show_contextmenu = False

    def save(self, widget, data):
        extracted = dict()
        for attr_name in itertools.chain(self.reserved_attrs, self.form_attrmap):
            value = data[attr_name].extracted
            if not value:
                continue
            extracted[attr_name] = value
        # possible values extracted by other user form behaviors
        extracted.update(self.model.attrs)
        users = ugm_backend.ugm.users
        user_id = extracted.pop('id')
        password = extracted.pop('password')
        login_name = general_settings(self.model).attrs.users_login_name_attr
        if login_name:
            extracted[login_name] = extracted.pop('login')
        users.create(user_id, **extracted)
        users()
        if self.model.local_manager_consider_for_user:
            groups = ugm_backend.ugm.groups
            for gid in self.model.local_manager_default_gids:
                groups[gid].add(user_id)
            groups()
        self.request.environ['next_resource'] = user_id
        if password is not UNSET:
            users.passwd(user_id, None, password)
        self.model.parent.invalidate()

    def next(self, request):
        next_resource = self.request.environ.get('next_resource')
        if next_resource:
            query = make_query(pid=next_resource)
            url = make_url(request.request, node=self.model.parent, query=query)
        else:
            url = make_url(request.request, node=self.model.parent)
        if self.ajax_request:
            return [
                AjaxAction(url, 'leftcolumn', 'replace', '.left_column'),
                AjaxAction(url, 'rightcolumn', 'replace', '.right_column'),
            ]
        return HTTPFound(location=url)


@tile(name='editform', interface=User, permission='edit_user', strict=False)
@plumbing(
    ContentEditForm,
    PrincipalRolesForm,
    PortraitForm,
    ExpirationForm,
    EditFormFiddle)
class UserEditForm(UserForm, Form):
    show_heading = False
    show_contextmenu = False

    def save(self, widget, data):
        attrs = self.model.attrs
        for attr_name in self.form_attrmap:
            if attr_name in ['id', 'password']:
                continue
            attrs[attr_name] = data[attr_name].extracted
        self.model()
        password = data.fetch('userform.password').extracted
        if password is not UNSET:
            user_id = self.model.name
            ugm_backend.ugm.users.passwd(user_id, None, password)

    def next(self, request):
        came_from = request.get('came_from')
        if came_from:
            came_from = compat.unquote(came_from)
            url = '{}?pid={}'.format(came_from, self.model.name)
        else:
            url = make_url(request.request, node=self.model)
        if self.ajax_request:
            return [
                AjaxAction(url, 'leftcolumn', 'replace', '.left_column'),
                AjaxAction(url, 'rightcolumn', 'replace', '.right_column'),
            ]
        return HTTPFound(location=url)
